package maca_ca;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import javax.naming.*;


public class CachedUser extends User {
  private BooleanCache cache = new BooleanCache();

  public CachedUser(LogWriter aLog, String serverAndPort, String userRoot,
              String userRDN, String ldap_pwd, contexts.ContextExpressionInterpreter anInterpreter) throws NamingException {
    super(aLog, serverAndPort, userRoot, userRDN, ldap_pwd, anInterpreter);
  }

  public CachedUser(DirContextHolder aContext, LogWriter aLog, String userRoot, String userRDN) throws NamingException,
                                                                                                       java.security.NoSuchAlgorithmException  {
    super(aContext, aLog, userRoot, userRDN);
  }

  public CachedUser(DirContextHolder aContext, String userRoot, String userRDN) throws NamingException,
                                                                                       java.security.NoSuchAlgorithmException  {
    super(aContext, userRoot, userRDN);
  }

  public CachedUser(String serverAndPort, String userRoot,
              String userRDN, String ldap_pwd) throws NamingException {
    super(serverAndPort, userRoot, userRDN, ldap_pwd);
  }

  public boolean hasAccess(String resName, String accessMode) {
    return this.hasAccess(resName, "", accessMode);
  }

  public boolean hasAccess(String resName, String params, String accessMode) {
   try {
      return cache.get(resName+params+accessMode);
    }
    catch (maca_ca.NotFoudInCacheException exc) {
      try {
        boolean result = super.hasAccess(resName, params, accessMode);
        cache.put(resName+params+accessMode, result);
        return result;
      }
      catch (Exception except) {
        return false;
      }
    }
    catch (Exception exc) {
      return false;
    }
  }

  public synchronized void resetCache() throws Exception {
    cache.resetCache();
  }



}