package maca_ca;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.lang.*;
import javax.swing.*;
import javax.naming.*;
import javax.swing.tree.*;
import javax.naming.directory.*;


public class Group implements TreeNode {
  //Constantes
  private static final DateFormat     DF = new SimpleDateFormat("dd/MM/yyyy 's' HH:mm:ss", new Locale("pt", "BR"));
  static protected final String LINE_END = System.getProperty("line.separator");

  protected DirContextHolder ctx = null;
  protected Attributes _attrs     = null;
  protected String _groupRoot;
  protected String _groupRDN;
  protected boolean newGroup = false;

  public void addNewGroup() throws NamingException {
    if ((ctx != null) && newGroup) {
        ctx.value.createSubcontext(this.getGroupDN(), this.getAttributesToBeSaved(this.getGroupAtributes()));
        this.updateGroupAtributes();
        newGroup = false;
    }
  }

  public boolean addMemberToGroup(String aUserDN) throws NamingException {
    if (aUserDN == null) {
      return false;
    }
    if (!this.hasUser(aUserDN, false)) {
      Attribute attr = new BasicAttribute(System.getProperty("maca_ca.ldap.groupMemberAttribute"),
                                          aUserDN);
      ModificationItem memberItem = new ModificationItem(DirContext.ADD_ATTRIBUTE, attr);
      try {
        ctx.value.modifyAttributes(this.getGroupDN(), new ModificationItem[] {memberItem});
      }
      catch (javax.naming.directory.SchemaViolationException exc) {
        //Primeiro membro no pode ser inserido, logo a classe de grupos tambm deve ser inserida.
        attr = new BasicAttribute("objectclass",
                                  System.getProperty("maca_ca.ldap.groupObjectClass"));
        ModificationItem memberClass = new ModificationItem(DirContext.ADD_ATTRIBUTE, attr);
        ctx.value.modifyAttributes(this.getGroupDN(), new ModificationItem[] {memberItem, memberClass});
      }
      return true;
    }
    return false;
  }

  public String resourceNameToLDAPresourceDN(String resName) {
    StringTokenizer strTk = new StringTokenizer(resName, ";");
    String resDN = "";
    while (strTk.hasMoreTokens()) {
      resDN = resDN + "cn="+strTk.nextToken()+",";
    }
    return resDN+System.getProperty("maca_ca.ldap.resourcesRDN")+","+System.getProperty("maca_ca.ldap.rootDN");
  }

  public String resourceNameToLDAPresourceDN(String resName, boolean reverse) {
    return this.resourceNameToLDAPresourceRDN(resName, reverse) + ","+System.getProperty("maca_ca.ldap.resourcesRDN")+","+System.getProperty("maca_ca.ldap.rootDN");
  }

  public String resourceNameToLDAPresourceRDN(String resName, boolean reverse) {
    StringTokenizer strTk = new StringTokenizer(resName, ";");
    String resDN = "";
    while (strTk.hasMoreTokens()) {
      if (reverse) {
        resDN = "cn="+strTk.nextToken()+(resDN.equalsIgnoreCase("")?"":",")+resDN;
      }
      else {
        resDN = resDN + "cn="+strTk.nextToken()+(strTk.hasMoreTokens()?",":"");
      }
    }
    return resDN;
  }

  public void ldapEntryToXML(String[] returningFields,
                             String subTreeScope,
                             boolean getMembers,
                             boolean indirectMembers,
                             Writer result) throws NamingException,
                                                                                                         IOException {
    if (returningFields == null) {
      returningFields = new String[0];
    }
    result.write("<ldapentry dn=\""+this.getGroupDN()+"\">" + LINE_END);
    Enumeration enumAttrs;
    if ((returningFields.length == 1) && ("*".equalsIgnoreCase(returningFields[0]))) {
      enumAttrs = this.getGroupAtributes().getAll();
    }
    else {
      Attributes      attrs = this.getGroupAtributes();
      Vector attrsList      = new Vector(returningFields.length);
      for (int i = 0; i < returningFields.length; i++) {
        Attribute attr = attrs.get(returningFields[i].trim());
        if (attr != null) {
          attrsList.add(attr);
        }
      }
      enumAttrs = new ObjectEnumeration(attrsList);
    }
    this.ldapAttributesToXML(enumAttrs, result);
    if (getMembers) {
      this.membersToXML(indirectMembers, result);
    }
    if ("onelevelscope".equalsIgnoreCase(subTreeScope) ||
        "subtreescope".equalsIgnoreCase(subTreeScope)) {
      String scope = ("onelevelscope".equalsIgnoreCase(subTreeScope))?"objectscope":subTreeScope;
      Enumeration enum = this.children();
      while (enum.hasMoreElements()) {
        Group grp = (Group) enum.nextElement();
        grp.ldapEntryToXML(returningFields, scope, getMembers, indirectMembers, result);
      }
    }
    result.write("</ldapentry>" + LINE_END);
  }

  public void membersToXML(boolean indirectMembers, Writer result) throws NamingException, IOException {
    NamingEnumeration answer = null;
    SearchControls      ctls = new SearchControls();
    if (indirectMembers) {
      ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    }
    else {
      ctls.setSearchScope(SearchControls.OBJECT_SCOPE);
    }
    String memberAttributeName = System.getProperty("maca_ca.ldap.groupMemberAttribute");
    ctls.setReturningAttributes(new String[] {memberAttributeName});
    answer = ctx.value.search(this.getGroupDN(), "(objectClass="+System.getProperty("maca_ca.ldap.groupObjectClass")+")", ctls);
    Hashtable keys = new Hashtable(3000);
    Attribute directMemberAttr = new BasicAttribute(memberAttributeName),
                    memberAttr = new BasicAttribute(memberAttributeName);
    while (answer.hasMore()) {
      SearchResult sr = (SearchResult) answer.next();
      Attribute  attr = sr.getAttributes().get(memberAttributeName);
      if (attr != null) {
        Enumeration enum = attr.getAll();
        while (enum.hasMoreElements()) {
          String member = (String) enum.nextElement();
          member = member.substring(member.indexOf("=")+1, member.indexOf(","));
          if (!keys.containsKey(member)) {
            if ("".equalsIgnoreCase(sr.getName())) { //Membros diretos
              directMemberAttr.add(member);
            }
            else { //Membros indiretos
              memberAttr.add(member);
            }
            keys.put(member, member);
          }
        }
      }
    }
    Vector vec = new Vector();
    result.write("<members type=\"direct\">" + LINE_END);
    if (directMemberAttr.size() > 0) {
      vec.add(directMemberAttr);
      this.ldapAttributesToXML(new ObjectEnumeration(vec), result);
    }
    result.write("</members>" + LINE_END);
    if (indirectMembers) {
      result.write("<members type=\"indirect\">" + LINE_END);
      if (memberAttr.size() > 0) {
        vec.clear();
        vec.add(memberAttr);
        this.ldapAttributesToXML(new ObjectEnumeration(vec), result);
      }
      result.write("</members>" + LINE_END);
    }
  }

  public String ldapResourceDNToResourceName(boolean reverse) throws NamingException {
    StringTokenizer strTk = new StringTokenizer(this.getGroupDN(), ",");
    String resName = "";
    while (strTk.hasMoreTokens()) {
      String nameComp = strTk.nextToken();
      if (nameComp.equalsIgnoreCase("cn=aplicacoes")) {
        if (reverse) {
          resName = nameComp.substring(nameComp.indexOf("=")+1, nameComp.length()) + (resName.equalsIgnoreCase("")?"":";") + resName;
        }
        else {
          resName = resName + nameComp.substring(nameComp.indexOf("=")+1, nameComp.length());
        }
        break;
      }
      else {
        if (reverse) {
          resName = nameComp.substring(nameComp.indexOf("=")+1, nameComp.length()) + (resName.equalsIgnoreCase("")?"":";") + resName;
        }
        else {
          resName = resName + nameComp.substring(nameComp.indexOf("=")+1, nameComp.length()) + (strTk.hasMoreTokens()?";":"");
        }
      }
    }
    return resName;
  }

  protected Enumeration getChildPathDNs(boolean relative) {
    Vector childPathDNs = new Vector();
    String parentRDN      = this.getParentRDN();
    Enumeration enum = this.children();
    while (enum.hasMoreElements()) {
      Group child = (Group)enum.nextElement();
      if (child != null) {
        childPathDNs.add((relative?(child.getGroupRDN()):(child.getGroupDN())));
        Enumeration enum2 = child.getChildPathDNs(relative);
        while (enum2.hasMoreElements()) {
          String childName = (String)enum2.nextElement();
          childPathDNs.add(childName);
        }
      }
    }
    return new ObjectEnumeration(childPathDNs);
  }

  protected Enumeration getParentsPathDNs(boolean relative) {
    Vector parentsPathDNs = new Vector();
    String parentRDN      = this.getParentRDN();
    while (!parentRDN.equalsIgnoreCase("")) {
      parentsPathDNs.add(parentRDN+(relative?"":","+this._groupRoot));
      int pos   = parentRDN.indexOf(",");
      if (pos == -1) {
        parentRDN = "";
      }
      else {
        parentRDN = parentRDN.substring(pos+1, parentRDN.length());
      }
    }
    return new ObjectEnumeration(parentsPathDNs);
  }

  public Group newGroup(DirContextHolder aContext, String groupRoot, String groupRDN) throws NamingException {
    return new Group(aContext, groupRoot, groupRDN);
  }

  public Vector getParentList() throws NamingException {
    Vector parentList = new Vector();
    Enumeration enum = this.getParentsPathDNs(true);
    while (enum.hasMoreElements()) {
      Group grp = this.newGroup(this.ctx, this._groupRoot, (String) enum.nextElement());
      if (grp != null) {
        parentList.add(grp);
      }
    }
    return parentList;
  }

  protected String getParentRDN() {
    String parentRDN = "";
    int          pos = _groupRDN.indexOf(",");
    if (pos == -1) {
      parentRDN = "";
    }
    else {
      parentRDN = _groupRDN.substring(pos+1,_groupRDN.length());
    }
    return parentRDN;
  }

  private void newGroupSetup(String groupStr) {
    if (this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.roleName").toUpperCase())) {
      this._attrs = new BasicAttributes(true);
      Attribute oc = new BasicAttribute("objectclass");
      oc.add("top");
      oc.add("incorrole");
      this._attrs.put(oc);
      int idx1 = groupStr.indexOf("=")+1;
      int idx2 = groupStr.indexOf(",");
      if (idx2 == -1) {
        this._attrs.put("cn",groupStr.substring(idx1, groupStr.length()));
      }
      else {
        this._attrs.put("cn",groupStr.substring(idx1, idx2));
      }
    }
    else if (this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.orgChartName").toUpperCase())) {
      this._attrs = new BasicAttributes(true);
      Attribute oc = new BasicAttribute("objectclass");
      oc.add("top");
      oc.add("organizationalunit");
      oc.add("inetorgperson");
      this._attrs.put(oc);
      int idx1 = groupStr.indexOf("=")+1;
      int idx2 = groupStr.indexOf(",");
      if (idx2 == -1) {
        this._attrs.put("cn",groupStr.substring(idx1, groupStr.length()));
        this._attrs.put("sn",groupStr.substring(idx1, groupStr.length()));
      }
      else {
        this._attrs.put("cn",groupStr.substring(idx1, idx2));
        this._attrs.put("sn",groupStr.substring(idx1, idx2));
      }
      this._attrs.put("ou","groups");
    }
    else { // Novo Recurso
      this._attrs = new BasicAttributes(true);
      Attribute oc = new BasicAttribute("objectclass");
      oc.add("top");
      oc.add("incorresource");
      this._attrs.put(oc);
      int idx1 = groupStr.indexOf("=")+1;
      int idx2 = groupStr.indexOf(",");
      if (idx2 == -1) {
        this._attrs.put("cn",groupStr.substring(idx1, groupStr.length()));
      }
      else {
        this._attrs.put("cn",groupStr.substring(idx1, idx2));
      }
      this._attrs.put("incorresourcetype","Aplicao");
      this._attrs.put("incorresourceaccessmode","Execuo");
    }
  }

  private void reOpen() throws NamingException {
      this.updateGroupAtributes();
  }

  private String getWellFormedUserDN(String uid) {
    if (!uid.trim().toLowerCase().startsWith("uid=")) {
      uid = "uid="+uid;
    }
    if (uid.trim().toLowerCase().endsWith((","+System.getProperty("maca_ca.ldap.peopleRDN").trim()).toLowerCase())) {
      uid = uid+","+System.getProperty("maca_ca.ldap.rootDN");
    }
    if (!uid.trim().toLowerCase().endsWith((","+
                                     System.getProperty("maca_ca.ldap.peopleRDN").trim()+","+
                                     System.getProperty("maca_ca.ldap.rootDN").trim()).toLowerCase())) {
      uid = uid+","+System.getProperty("maca_ca.ldap.peopleRDN")+","+
                    System.getProperty("maca_ca.ldap.rootDN");
    }
    return uid;
  }

  private Attributes getAttributesToBeSaved(Attributes attrs) throws NamingException  {
    Attributes nonEmptyAttrs = (Attributes) attrs.clone();
    NamingEnumeration   enum = nonEmptyAttrs.getIDs();
    while (enum.hasMoreElements()) {
      String attrID = (String) enum.nextElement();
      Attribute attr = nonEmptyAttrs.get(attrID);
      if ((attr != null) && attr.get().equals("")){
        nonEmptyAttrs.remove(attrID);
      }
    }
    if (this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.organograma"))) {
      Attribute attr = nonEmptyAttrs.get("cn");
      if (attr != null) {
        nonEmptyAttrs.put("sn", attr.get());
      }
    }
    return nonEmptyAttrs;
  }


  private void ldapAttributesToXML(Enumeration enumAttrs, Writer result) throws NamingException, IOException {
    while ((enumAttrs != null) && (enumAttrs.hasMoreElements())) {
      Attribute attr = (Attribute) enumAttrs.nextElement();
      if ((attr != null) && (!attr.getID().equalsIgnoreCase("userPassword"))) {
        result.write("<attribute name=\""+attr.getID()+"\">" + LINE_END);
        Enumeration enum = attr.getAll();
        while (enum.hasMoreElements()) {
          Object value = enum.nextElement();
          if (value != null) {
            result.write("<value><![CDATA["+value+"]]></value>" + LINE_END);
          }
        }
        result.write("</attribute>" + LINE_END);
      }
    }
  }


  private ModificationItem[] getAttributesToBeModified(Attributes attrs) throws NamingException  {
    Attributes currentAttrs = ctx.value.getAttributes(this.getGroupDN());
    Attribute oc            = currentAttrs.get("objectclass");
    if (this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.organograma"))) {
      if (!oc.contains("inetorgperson")) {
        oc.add("inetorgperson");
      }
      Attribute attr = attrs.get("cn");
      if (attr != null) {
        attrs.put("sn", attr.get());
      }
    }
    Vector  modItems = new Vector();
    modItems.add(new ModificationItem(DirContext.REPLACE_ATTRIBUTE, oc));
    NamingEnumeration  enum = attrs.getIDs();
    while (enum.hasMoreElements()) {
      String attrID = (String) enum.nextElement();
      Attribute attr = attrs.get(attrID);
      if ((attr != null) && !attrID.equalsIgnoreCase("objectclass")) {
        if (!attr.get().equals("")) {
          if (currentAttrs.get(attrID) == null) { // Adiciona novo atributo
            modItems.add(new ModificationItem(DirContext.ADD_ATTRIBUTE, attr));
          }
          else { // Modifica atributo existente
            modItems.add(new ModificationItem(DirContext.REPLACE_ATTRIBUTE, attr));
          }
        }
        else if (currentAttrs.get(attrID) != null) { // Remove atributo existente
          modItems.add(new ModificationItem(DirContext.REMOVE_ATTRIBUTE, currentAttrs.get(attrID)));
        }
      }
    }
    ModificationItem[] mods = new ModificationItem[modItems.size()];
    for (int n = 0; n < mods.length; n++) {
      mods[n] = (ModificationItem) modItems.get(n);
    }
    return mods;
  }

  private boolean isGroup(String groupStr) throws NamingException {
    SearchControls      ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.OBJECT_SCOPE);
    ctls.setReturningAttributes(new String[] {""});
    try {
      NamingEnumeration answer = ctx.value.search(groupStr+","+this._groupRoot, "(objectclass="+System.getProperty("maca_ca.ldap.groupObjectClass")+")", ctls);
    }
    catch (javax.naming.NameNotFoundException exc) {
      return false;
    }
    return true;
  }

  public void renameGroupName(String newGroupName) throws NamingException {
    if ((newGroupName == null) || newGroupName.equalsIgnoreCase("") || this.newGroup) {
      throw new NamingException("Novo nome do grupo no pode ser vazio!");
    }
    if (!this.isLeaf()) {
        throw new NamingException("Grupo selecionado possui subgrupos.\n"+
                                   "NO  possivel renome-lo!");
    }
    if (this.getName().equalsIgnoreCase("aplicacoes")) {
      throw new NamingException("No se pode renomear o recurso aplicacoes!");
    }
    if (this.getGroupDN().equalsIgnoreCase("cn=administrador,"+
                                           System.getProperty("maca_ca.ldap.roleRDN")+","+
                                           System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                           System.getProperty("maca_ca.ldap.rootDN"))) {
      throw new NamingException("No se pode renomear o papel administrador!");
    }
    if (this.getName().equalsIgnoreCase(this.getGroupType())) {
      throw new NamingException("No se pode renomear o grupo "+this.getGroupType()+"!");
    }
    if (this.hasAuthorization()) {
      throw new NamingException("No se pode renomear um recurso com autorizaes associadas!");
    }
    ctx.value.rename(this.getGroupDN(), "cn="+newGroupName+","+
                                              this.getParentRDN()+
                                              (this.getParentRDN().equalsIgnoreCase("")?"":",")+
                                              this._groupRoot);
    this._groupRDN = "cn="+newGroupName+(this.getParentRDN().equalsIgnoreCase("")?"":",")+this.getParentRDN();
    this._attrs.put("cn", newGroupName);
    this.saveGroupAtributes();
  }

  public boolean removeGroup(boolean removeSubTree) throws NamingException {
    boolean done = false;
    if ((this.ctx != null) && (!newGroup)) {
      if (removeSubTree) {
        //ctx.destroySubcontext(_groupRDN);
      }
      else {
        if (this.isLeaf()) {
          if (!this.hasMembers(false)) {
            if (!this.getName().equalsIgnoreCase(System.getProperty("maca_ca.orgChartName")) &&
                !this.getName().equalsIgnoreCase(System.getProperty("maca_ca.roleName"))) {
              if (!this.getName().equalsIgnoreCase("administrador")) {
                if (!this.hasAuthorization()) {
                  if (this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.roleName"))) {
                    Role  role = this.getGroupAsRole();
                    this.ctx.value.destroySubcontext(role.getRoleAuthorizationRoot());
                  }
                  this.ctx.value.destroySubcontext(this.getGroupDN());
                  done = true;
                }
                else {
                  JOptionPane.showMessageDialog(null, "Grupo selecionado possui autorizaes.\n"+
                                                "NO  possvel remov-lo!",
                                                "Informao",
                                                JOptionPane.INFORMATION_MESSAGE);
                }
              }
              else {
                JOptionPane.showMessageDialog(null, "No  permitido remover o\n"+
                                              "papel de ADMINISTRADOR!",
                                              "Informao",
                                              JOptionPane.INFORMATION_MESSAGE);
              }
            }
            else {
              JOptionPane.showMessageDialog(null, "No  permitido remover os\n"+
                                            "grupos "+System.getProperty("maca_ca.orgChartName").toUpperCase()+
                                            " e "+System.getProperty("maca_ca.roleName").toUpperCase()+"!",
                                            "Informao",
                                            JOptionPane.INFORMATION_MESSAGE);
            }
          }
          else {
            JOptionPane.showMessageDialog(null, "Grupo selecionado possui membros.\n"+
                                          "NO  possvel remov-lo!",
                                          "Informao",
                                          JOptionPane.INFORMATION_MESSAGE);
          }
        }
        else {
          JOptionPane.showMessageDialog(null, "Grupo selecionado possui subgrupos.\n"+
                                        "NO  possivel remov-lo!",
                                        "Informao",
                                        JOptionPane.INFORMATION_MESSAGE);
        }
      }
    }
    if ((this.ctx != null) && done) {
      this.ctx = null;
    }
    return done;
  }

  public boolean removeMemberFromGroup(String aUserDN) throws NamingException {
    if (aUserDN == null) {
      return false;
    }
    if (this.hasUser(aUserDN, false)) {
      Attribute attr = new BasicAttribute(System.getProperty("maca_ca.ldap.groupMemberAttribute"),
                                          aUserDN);
      ModificationItem memberItem = new ModificationItem(DirContext.REMOVE_ATTRIBUTE, attr);
      try {
        ctx.value.modifyAttributes(this.getGroupDN(), new ModificationItem[] {memberItem});
      }
      catch (javax.naming.directory.SchemaViolationException exc) {
        //ltimo membro no pode ser removido, logo a classe de grupos tambm deve ser removida.
        attr = new BasicAttribute("objectclass",
                                  System.getProperty("maca_ca.ldap.groupObjectClass"));
        ModificationItem memberClass = new ModificationItem(DirContext.REMOVE_ATTRIBUTE, attr);
        ctx.value.modifyAttributes(this.getGroupDN(), new ModificationItem[] {memberItem, memberClass});
      }
      return true;
    }
    return false;
  }

  public boolean hasUser(String userID, boolean subTreeScope) throws NamingException {
    NamingEnumeration      answer = null;
    Vector         rolesNamesList = new Vector();
    SearchControls ctls = new SearchControls();
    if (subTreeScope) {
      ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    }
    else {
      ctls.setSearchScope(SearchControls.OBJECT_SCOPE);
    }
    ctls.setReturningAttributes(new String[] {"cn"});
    answer = ctx.value.search(this.getGroupDN(),
                              "("+System.getProperty("maca_ca.ldap.groupMemberAttribute")+"="+
                              this.getWellFormedUserDN(userID)+")",
                              ctls);
    return answer.hasMoreElements();
  }

  public Attributes getGroupAtributes() throws NamingException {
    if (this._attrs == null) {
      this.updateGroupAtributes();
    }
    return this._attrs;
  }

  protected void updateGroupAtributes() throws NamingException {
      if (this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.roleName")) ||
          this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.orgChartName"))) {
        Attribute attr  = ctx.value.getAttributes(this.getGroupDN(), new String[] {"objectclass"}).get("objectclass");
        if (attr != null) {
          boolean hasMembers     = false;
          NamingEnumeration enum = attr.getAll();
          while (enum.hasMoreElements()) {
            Object val = enum.nextElement();
            if ((val != null) && (val instanceof String)) {
              if (((String) val).equalsIgnoreCase(System.getProperty("maca_ca.ldap.groupObjectClass"))) {
                hasMembers = true;
                break;
              }
            }
          }
          if (!hasMembers) {
            this._attrs  = ctx.value.getAttributes(this.getGroupDN());
          }
          else {
            String[] attrsids = new String[] {"objectclass", "cn", "sn", "ou", "description",
                                              "initials", "manager", "l", "telephonenumber",
                                              "facsimiletelephonenumber", "mail", "labeleduri"};
            this._attrs  = ctx.value.getAttributes(this.getGroupDN(), attrsids);
          }
        }
        else {
          this._attrs  = ctx.value.getAttributes(this.getGroupDN());
        }
      }
      else {
        this._attrs  = ctx.value.getAttributes(this.getGroupDN());
      }
  }

  public void saveGroupAtributes() throws NamingException {
    if ((ctx != null) && (!newGroup)) {
        ctx.value.modifyAttributes(this.getGroupDN(), this.getAttributesToBeModified(this.getGroupAtributes()));
        this.updateGroupAtributes();
    }
    else {
      this.addNewGroup();
    }
  }

  public Group getChildWith(String roleName) {
    if (this.getName().equalsIgnoreCase(roleName)) {
      return this;
    }
    else {
      if (this.isLeaf()) {
        return null;
      }
      else {
        Enumeration childrenEnum = this.children();
        Group                grp = null;
        while (childrenEnum.hasMoreElements()) {
          Group gp = (Group) childrenEnum.nextElement();
          grp = gp.getChildWith(roleName);
          if (grp != null) {
            break;
          }
        }
        return grp;
      }
    }
  }

  protected Group() {

  }

  public Group(DirContextHolder aContext, String groupRoot, String groupRDN)  throws NamingException  {

    this.ctx              = aContext;
    this._groupRoot       = groupRoot;
    this._groupRDN        = groupRDN;
    this.newGroup         = false;
  }

  public Group(DirContextHolder aContext, String groupRoot, String groupRDN, boolean newGrp) throws NamingException {

    this.ctx              = aContext;
    this._groupRoot       = groupRoot;
    this._groupRDN        = groupRDN;
    this.newGroup         = newGrp;
    if (newGroup) {
      if ((!_groupRDN.equalsIgnoreCase("")) && (!this.isGroup(_groupRDN))) {
        newGroup = true;
        this.newGroupSetup(_groupRDN);
      }
      else {
        throw new NamingException("Nome do novo grupo '"+this.getGroupDN()+"' j existe ou  vazio!");
      }
    }
    else {
      newGroup = false;
    }
  }

  public int getLevel() {
    StringTokenizer strTk = new StringTokenizer(_groupRDN, ",");
    return strTk.countTokens();
  }

  public boolean hasAuthorization() throws NamingException {
    NamingEnumeration answer = null;
    SearchControls ctls = new SearchControls();
    String searchString = "";
    if (this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.roleName"))) {
      searchString = "(incorroledn="+this.getGroupDN()+")";
    }
    else if (this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.resourcesName"))) {
      searchString = "(incorresourcedn="+this.getGroupDN()+")";
    }
    else {
      return false;
    }
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    answer = ctx.value.search(System.getProperty("maca_ca.ldap.authorizationsRDN")+","+System.getProperty("maca_ca.ldap.rootDN"), searchString, ctls);
    return answer.hasMore();
  }

  public boolean hasMembers(boolean subTree)  throws NamingException {
    if (ctx == null) {
      return false;
    }
    NamingEnumeration answer = null;
    SearchControls ctls = new SearchControls();

    if (subTree) {
      ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    }
    else {
      ctls.setSearchScope(SearchControls.OBJECT_SCOPE);
    }
    ctls.setReturningAttributes(new String[] {System.getProperty("cn")});
    answer = ctx.value.search(this.getGroupDN(), "(objectClass="+System.getProperty("maca_ca.ldap.groupObjectClass")+")", ctls);
    return answer.hasMore();
  }

  public Vector getMembersIDS(boolean subTreeScope)  throws NamingException {
    NamingEnumeration answer = null;
    Vector      groupsList   = new Vector();
    SearchControls ctls = new SearchControls();

    if (ctx == null) {
      return groupsList;
    }

    if (subTreeScope) {
      ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    }
    else {
      ctls.setSearchScope(SearchControls.OBJECT_SCOPE);
    }
    ctls.setReturningAttributes(new String[] {System.getProperty("maca_ca.ldap.groupMemberAttribute")});
    answer = ctx.value.search(this.getGroupDN(), "(objectClass="+System.getProperty("maca_ca.ldap.groupObjectClass")+")", ctls);
    while (answer.hasMore()) {
      SearchResult sr = (SearchResult) answer.next();
      Attributes attrs = sr.getAttributes();
      Attribute  attr  = attrs.get(System.getProperty("maca_ca.ldap.groupMemberAttribute"));
      if (attr != null) {
        NamingEnumeration enum = attr.getAll();
        while (enum.hasMore()) {
          String userID = (String) enum.next();
          userID = userID.substring(userID.indexOf("=")+1, userID.indexOf(","));
          if (!groupsList.contains(userID)) {
            groupsList.add(userID);
          }
        }
      }
    }
    return groupsList;
  }

  public int getMembersToXML(String filter,
                             String[] returningAttrs,
                             boolean getRoles,
                             boolean tabbed,
                             Writer result) throws Exception {
    int           membersNum = 0;
    NamingEnumeration answer = null;
    if (ctx != null) {
      SearchControls ctls = new SearchControls();
      ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
      ctls.setReturningAttributes(returningAttrs);
      if (filter.equalsIgnoreCase("")) {
        filter = "uid=*";
      }
      String rolesRootDN = System.getProperty("maca_ca.ldap.roleRDN")+","+
                           System.getProperty("maca_ca.ldap.groupsRDN")+","+
                           System.getProperty("maca_ca.ldap.rootDN");
      String userRootDN = this.getGroupDN().trim();
      String  groupType = "user";
      answer = ctx.value.search(userRootDN, "("+filter+")", ctls);
      while (answer.hasMore()) {
        SearchResult sr = (SearchResult) answer.next();
        User user = new User(ctx, userRootDN, sr.getName().trim());
        String userDN = user.getUserDN();
        String lockTime = "";
        boolean locked  = user.isAccountLocked();
        if (locked) {
          Date time = user.getPasswordAttributeDate("passwordlocktime");
          if (time != null) {
          	lockTime = DF.format(time);
          }
        }
        else {
          Date time = user.getPasswordAttributeDate("passwordunlocktime");
          if (time != null) {
          	lockTime = DF.format(time);
          }
        }
        result.write("<"+groupType+" accountLocked=\""+locked+"\" lockTime=\""+lockTime+"\">" + LINE_END); 
        result.write("<ldapentry dn=\""+userDN+"\">" + LINE_END);
        Attributes allAttrs = new BasicAttributes(true);
        if ((returningAttrs.length == 1) && (returningAttrs[0].trim().equals("*"))) {
           returningAttrs = UserSchema.getAttributesNames();
        }
        for (int i = 0; i < returningAttrs.length; i++) {
          if (!"*".equalsIgnoreCase(returningAttrs[i])) {
            Attributes attrSchema = UserSchema.getAttributeSchema(returningAttrs[i]);
            if ((attrSchema != null) && (!"objectclass".equalsIgnoreCase(returningAttrs[i]))) {
              Attribute newAttr = new BasicAttribute((String) returningAttrs[i]);
              Attribute    attr = attrSchema.get("default");
              if (attr != null) {
                try {
                  javax.naming.NamingEnumeration enum = attr.getAll();
                  while (enum.hasMoreElements()) {
                    Object value = enum.nextElement();
                    if (value != null) {
                      newAttr.add(value);
                    }
                  }
                }
                catch (javax.naming.NamingException exc) {}
              }
              else {
                newAttr.add("");
              }
              allAttrs.put(newAttr);
            }
          }
        }
        NamingEnumeration attrsEnum = sr.getAttributes().getAll();
        while (attrsEnum.hasMoreElements()) {
          allAttrs.put((Attribute) attrsEnum.nextElement());
        }
        User.ldapAttributesToXML(allAttrs, result, tabbed);
        result.write("</ldapentry>" + LINE_END);
        if (getRoles) {
          result.write("<roles>" + LINE_END);
          ctls.setReturningAttributes(new String[] {"cn"});
          NamingEnumeration rolesAnswer =
              ctx.value.search(rolesRootDN,
                               "("+System.getProperty("maca_ca.ldap.groupMemberAttribute")+"="+userDN+")",
                               ctls);
          while (rolesAnswer.hasMore()) {
            SearchResult rolesSr  = (SearchResult) rolesAnswer.next();
            String roleDN = rolesSr.getName().trim()  + "," +  rolesRootDN;
            result.write("<ldapentry dn=\""+roleDN+"\">" + LINE_END);
            Attributes rolesAttrs = rolesSr.getAttributes();
            this.ldapAttributesToXML(rolesAttrs.getAll(), result);
            result.write("</ldapentry>" + LINE_END);
          }
          result.write("</roles>" + LINE_END);
        }
        result.write("</"+groupType+">" + LINE_END);
        membersNum += 1;
      }
    }
    return membersNum;
  }

  public Vector getMembers(String filter, Vector returningAttrsVector) throws NamingException {
    String[] returningAttrs  = new String[returningAttrsVector.size()];
    for (int n = 0; n < returningAttrsVector.size(); n++) {
      returningAttrs[n] = (String) returningAttrsVector.get(n);
    }
    return this.getMembers(filter, returningAttrs);
  }

  public Vector getMembers(String filter, String[] returningAttrs) throws NamingException {
    NamingEnumeration answer = null;
    Vector           members = new Vector();
    if (ctx == null) {
      return members;
    }
    SearchControls ctls = new SearchControls();

    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);

    ctls.setReturningAttributes(returningAttrs);
    if (filter.equalsIgnoreCase("")) {
      filter = "uid=*";
    }
    answer = ctx.value.search(this.getGroupDN(), "("+filter+")", ctls);
    while (answer.hasMore()) {
      SearchResult sr = (SearchResult) answer.next();
      Attributes attrs = sr.getAttributes();
      Vector memberAttrs = new Vector();
      for (int n = 0; n < returningAttrs.length; n++) {
        if (returningAttrs[n] != null) {
          Attribute  attr  = attrs.get(returningAttrs[n]);
          if ((attr != null) &&
              (attr.get() != null))  {
            memberAttrs.add(attr.get());
          }
          else {
            memberAttrs.add("");
          }
        }
      }
      members.add(memberAttrs);
    }
    return members;
  }

  public String getName() {
    String name = "";
    int  posIni = _groupRDN.indexOf("=");

    if (posIni == -1) {
      name = "\\";
    }
    else {
      int  posFin = _groupRDN.indexOf(",");
      if (posFin == -1) {
        name = _groupRDN.substring(posIni+1,_groupRDN.length());
      }
      else {
        name = _groupRDN.substring(posIni+1,posFin);
      }
    }
    return name;
  }

  public String getGroupType() {
    String name = "";
    if (_groupRDN.toUpperCase().indexOf((System.getProperty("maca_ca.ldap.roleRDN")).toUpperCase()) != -1) {
      name = System.getProperty("maca_ca.roleName");
    }
    else if (_groupRDN.toUpperCase().indexOf((System.getProperty("maca_ca.ldap.orgChartRDN")).toUpperCase()) != -1) {
      name = System.getProperty("maca_ca.orgChartName");
    }
    else  if (_groupRoot.toUpperCase().indexOf((System.getProperty("maca_ca.ldap.resourcesRDN")).toUpperCase()) != -1) {
      name = System.getProperty("maca_ca.resourcesName");
    }
    else  if (_groupRoot.toUpperCase().indexOf((System.getProperty("maca_ca.ldap.peopleRDN")).toUpperCase()) != -1) {
      name = System.getProperty("maca_ca.peopleName");
    }
    else {
      name = "";
    }
    return name;
  }

  public String getGroupDN() {
    String groupRDN;

    groupRDN = (_groupRDN=="")?_groupRDN:_groupRDN+",";
    return groupRDN+_groupRoot;
  }

  public String getGroupRDN() {
    return this._groupRDN;
  }

  public String getGroupRoot() {
    return this._groupRoot;
  }


  public Role getGroupAsRole() throws NamingException {
    if (this.getGroupType().equalsIgnoreCase(System.getProperty("maca_ca.roleName"))) {
       if (this.getName().equalsIgnoreCase(System.getProperty("maca_ca.roleName"))) {
         return new Role(this.ctx, this._groupRoot, "cn="+this.getGroupType());
       }
       else {
         return new Role(this.ctx, "cn="+this.getGroupType()+","+this._groupRoot,
                         this._groupRDN.substring(0, this._groupRDN.toUpperCase().indexOf("CN="+this.getGroupType().toUpperCase())-1));
       }
    }
    else {
      return null;
    }
  }

  public Enumeration children() {
    NamingEnumeration answer = null;
    ObjectEnumeration  groups = null;
    Vector        groupsList = new Vector();

    if (ctx == null) {
      return new ObjectEnumeration(groupsList);
    }

    SearchControls ctls = new SearchControls();

    ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    try {
      answer = ctx.value.search(this.getGroupDN(), "(cn=*)", ctls);
      while (answer.hasMore()) {
        SearchResult sr = (SearchResult) answer.next();
        if (_groupRDN == "") {
          groupsList.add(new Group(ctx, _groupRoot, sr.getName()));
        }
        else {
          groupsList.add(new Group(ctx, _groupRoot, sr.getName()+","+_groupRDN));
        }
      }
    }
    catch (NamingException exc) {
      exc.printStackTrace();
    }
    groups = new ObjectEnumeration(groupsList);
    return groups;
  }

  public TreeNode getParent() {
    TreeNode tn = null;
    try {
      if (_groupRDN == "") {
        tn = null;
      }
      else {
        tn = new Group(ctx, _groupRoot, getParentRDN());
      }
    }
    catch (NamingException exc) {
      exc.printStackTrace();
    }
    return tn;
  }

  public TreeNode getChildAt(int index) {
    NamingEnumeration answer = null;
    Group             group  = null;
    SearchControls ctls      = new SearchControls();

    if (ctx == null) {
      return group;
    }

    ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    try {
      answer = ctx.value.search(this.getGroupDN(), "(cn=*)", ctls);
      int n = 0;
      while (answer.hasMore()) {
        SearchResult sr = (SearchResult) answer.next();
        if (n == index) {
          if (_groupRDN == "") {
            group = new Group(ctx, _groupRoot, sr.getName());
          }
          else {
            group = new Group(ctx, _groupRoot, sr.getName()+","+_groupRDN);
          }
          break;
        }
        n = n + 1;
      }
    }
    catch (NamingException exc) {
      exc.printStackTrace();
    }
    return group;
  }

  public int getChildCount() {
    if (ctx == null) {
      return 0;
    }
    NamingEnumeration answer = null;
    SearchControls ctls      = new SearchControls();
    int                    n = 0;

    ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    try {
      answer = ctx.value.search(this.getGroupDN(), "(cn=*)", ctls);
      while (answer.hasMore()) {
        answer.next();
        n = n + 1;
      }
    }
    catch (NamingException exc) {
      exc.printStackTrace();
    }
    return n;
  }

  public int getIndex(TreeNode treeNode) {
    NamingEnumeration answer = null;
    SearchControls      ctls = new SearchControls();
    int                    n = 0;
    int              foundAt = -1;

    if (ctx == null) {
      return foundAt;
    }

    Group            grpNode = (Group) treeNode;
    if (treeNode != null) {
      ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
      ctls.setReturningAttributes(new String[] {"cn"});
      try {
        answer = ctx.value.search(this.getGroupDN(), "(cn=*)", ctls);
        while (answer.hasMore()) {
          SearchResult sr = (SearchResult) answer.next();
          Attributes attrs = sr.getAttributes();
          Attribute   attr = attrs.get("cn");
          if (attr != null) {
            String attrValue = (String)attr.get();
            if (attrValue.equalsIgnoreCase(grpNode.getName())) {
              foundAt = n;
              break;
            }
          }
          n = n + 1;
        }
      }
      catch (NamingException exc) {
        System.out.println(exc);
      }
    }
    return foundAt;
  }

  protected Vector getAllGroupsRDN() throws NamingException  {
    Vector     namings = new Vector();
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    NamingEnumeration answer = ctx.value.search("cn="+this.getGroupType()+",ou=groups"+","+System.getProperty("maca_ca.ldap.rootDN"), "cn=*", ctls);
    answer.next();
    while (answer.hasMore()) {
      SearchResult sr  = (SearchResult) answer.next();
      namings.add(sr.getName());
    }
    return namings;
  }

  public boolean getAllowsChildren() {
    return true;
  }

  public boolean isRoot(){
    return (this.getLevel() == 0);
  }

  public boolean isNewGroup(){
    return this.newGroup;
  }

  public boolean isLeaf(){
    return getChildCount() == 0;
  }

  public boolean isNew(){
    return this.newGroup;
  }


  public String toString() {
    return getName();
  }

}