package maca_ca.ssl;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;
import javax.net.ssl.*;
import java.security.*;
import java.security.cert.*;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;


public class X509Truster implements X509TrustManager {
  private static final DateFormat          DF = new SimpleDateFormat("E, dd/MM/yyyy 'as' HH:mm:ss", new Locale("pt", "BR"));
  private static final int CA_SESSION_TRUSTED = 0;
  private static final int  CA_ALWYAS_TRUSTED = 1;
  private static final int     CA_NOT_TRUSTED = 2;

  private TrustManagerFactory                  tmf = null;
  private X509TrustManager     sunX509TrustManager = null;
  private X509TrustManager maca_caX509TrustManager = null;
  private KeyStore                              ks = null;

  private static String getPassword(String keyStoreFile) {
    String pwd = System.getProperty("maca_ca.ldap.ssl.keystorepwd");
    if ((pwd == null) || "".equalsIgnoreCase(pwd)) {
      if ("false".equalsIgnoreCase(System.getProperty("maca_ca.ldap.ssl.console"))) {
        javax.swing.JTextField pwdField = new JPasswordField(30);
        Object comp[] = new Object[3];
        comp[0] = "Keystore: " + keyStoreFile;
        comp[1] = "Entre com a senha: ";
        comp[2] = pwdField;
        Object options[] = { "OK" };
        pwd = null;
        while ((pwd == null) || "".equals(pwd)) {
          JOptionPane.showOptionDialog(null, ((Object) (comp)), "Senha do Keystore", -1, 3, null, options, options[0]);
          pwd = pwdField.getText().trim();
        }
      }
      else {
        PasswordField prompt = new PasswordField();
        System.out.println("Keystore: " + keyStoreFile);
        try {
          while ((pwd == null) || "".equals(pwd)) {
            pwd = prompt.getPassword("Entre com a senha: ").trim();
          }
          System.out.println();
        }
        catch (IOException exc) {
          exc.printStackTrace();
        }
      }
      System.setProperty("maca_ca.ldap.ssl.keystorepwd", pwd);
    }
    return pwd;
  }

  private void initKeyStore(String keystoreFile, String pwd) {
    try {
      if (keystoreFile.trim().toLowerCase().endsWith(".p12")) {
           ks = KeyStore.getInstance("PKCS12");
      }
      else {
            ks = KeyStore.getInstance("JKS");
      }
      InputStream in = null;
      if (keystoreFile.indexOf("://") == -1) {
        try {
          in = new FileInputStream(keystoreFile);
        }
        catch (FileNotFoundException exc) {
        }
      }
      else {
       try {
         URL url = new URL(keystoreFile);
         URLConnection con = url.openConnection();
         in = con.getInputStream();
        }
        catch (MalformedURLException exc) {
          System.out.println("maca_ca.ssl.X509Truster - URL para localizar o keystore invlida: " + exc.getMessage());
          throw exc;
        }
        catch (IOException exc) {
        }
      }
      try {
        ks.load(in, pwd.toCharArray());
      }
      finally {
        if (in != null) {
          in.close();
        }
      }
    }
    catch (Exception exc) {
      System.out.println("maca_ca.ssl.X509Truster - falha criando ou lendo o keystore: " + exc.getMessage());
    }
  }

  private int certificateAccepted(X509Certificate[] chain) {
    X509Certificate caCert = chain[chain.length - 1];
    if ("false".equalsIgnoreCase(System.getProperty("maca_ca.ldap.ssl.console"))) {
      String msg  = "Voc confia neste certificado:\n";
             msg += "    Emitido para: "+caCert.getSubjectDN().getName() + "\n";
             msg += "     Emitido por: "+caCert.getIssuerDN().getName() + "\n";
             msg += "       Vlido de: "+DF.format(caCert.getNotBefore()) + "\n";
             msg += "      Vlido at: "+DF.format(caCert.getNotAfter()) + "\n";
      Object options[] = {"Nesta sesso apenas", "Sempre", "No"};
      return JOptionPane.showOptionDialog(null, ((Object) (msg)), "O certificado da AC do servidor  desconhecido.", -1, 2, null, options, options[0]);
    }
    else {
      System.out.println("O certificado da AC do servidor  desconhecido.");
      System.out.println("Voc confia neste certificado:");
      System.out.println("  Emitido para: "+caCert.getSubjectDN().getName());
      System.out.println("   Emitido por: "+caCert.getIssuerDN().getName());
      System.out.println("     Valido de: "+DF.format(caCert.getNotBefore()));
      System.out.println("    Valido ate: "+DF.format(caCert.getNotAfter()));
      System.out.print("Nesta sessao apenas(s), sempre(p) ou nao(n): ");
      try {
        char c = (char)System.in.read();
        if (Character.toLowerCase(c) == 's') {
          return CA_SESSION_TRUSTED;
        }
        else if (Character.toLowerCase(c) == 'p') {
          return CA_ALWYAS_TRUSTED;
        }
      }
      catch (Exception exc) {
        exc.printStackTrace();
      }
    }
    return CA_NOT_TRUSTED;
  }

  private void setCASessionTrusted(X509Certificate[] chain) throws CertificateException {
    try {
      X509Certificate caCert = chain[chain.length - 1];
      ks.setCertificateEntry(String.valueOf(System.currentTimeMillis()), caCert);
      tmf.init(ks);
      maca_caX509TrustManager = (X509TrustManager) tmf.getTrustManagers()[0];
    }
    catch (Exception exc) {
      throw new CertificateException(exc.getMessage());
    }
  }

  private void setCAlwaysTrusted(X509Certificate[] chain) throws CertificateException {
    try {
      setCASessionTrusted(chain);
      String keystoreFile = System.getProperty("maca_ca.ldap.ssl.keystore");
      OutputStream out = null;
      if (keystoreFile.indexOf("://") == -1) {
        out = new FileOutputStream(keystoreFile);
      }
      else {
        URL url = new URL(keystoreFile);
        URLConnection con = url.openConnection();
        con.setDoOutput(true);
        out = con.getOutputStream();
      }
      try {
        ks.store(out, this.getPassword(keystoreFile).toCharArray());
      }
      finally {
        if (out != null) {
          out.close();
        }
      }
    }
    catch (Exception exc) {
      throw new CertificateException(exc.getMessage());
    }
  }

  public X509Truster() {
    try {
      initKeyStore(System.getProperty("maca_ca.ldap.ssl.keystore"), this.getPassword(System.getProperty("maca_ca.ldap.ssl.keystore")));
      tmf = TrustManagerFactory.getInstance("SunX509", "SunJSSE");
      tmf.init((KeyStore) null);
      sunX509TrustManager = (X509TrustManager) tmf.getTrustManagers()[0];
      tmf.init(ks);
      maca_caX509TrustManager = (X509TrustManager) tmf.getTrustManagers()[0];
    }
    catch (Exception exc) {
      exc.printStackTrace();
    }
  }

  public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
    throw new CertificateException("Autenticao do cliente no  suportada");
  }

  public void checkServerTrusted(X509Certificate[] chain,  String authType)  throws CertificateException {
    try {
      try {
        try {
          maca_caX509TrustManager.checkServerTrusted(chain, authType);
        }
        catch (CertificateException except) {
          sunX509TrustManager.checkServerTrusted(chain, authType);
        }
      }
      catch (CertificateException exc) {
        int result = this.certificateAccepted(chain);
        if (result == this.CA_SESSION_TRUSTED) {
          //Confiar agora
          setCASessionTrusted(chain);
        }
        else if (result == this.CA_ALWYAS_TRUSTED) {
          //Confiar sempre
          setCAlwaysTrusted(chain);
        }
        else {
          //No confiar
          throw exc;
        }
      }
    }
    catch (Exception exc) {
      if (exc instanceof CertificateException) {
        throw (CertificateException) exc;
      }
      throw new CertificateException(exc.getMessage());
    }
  }

  public X509Certificate[] getAcceptedIssuers() {
    X509Certificate[] cas = new X509Certificate[sunX509TrustManager.getAcceptedIssuers().length +
                                                maca_caX509TrustManager.getAcceptedIssuers().length];
    for (int i = 0; i < sunX509TrustManager.getAcceptedIssuers().length; i++) {
      cas[i] = sunX509TrustManager.getAcceptedIssuers()[i];
    }
    for (int i = 0; i < maca_caX509TrustManager.getAcceptedIssuers().length; i++) {
      cas[sunX509TrustManager.getAcceptedIssuers().length + i] = maca_caX509TrustManager.getAcceptedIssuers()[i];
    }
    return cas;
  }

}