package maca_ad;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import maca_ca.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import tablesorter.*;
import javax.naming.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.JOptionPane;
import javax.naming.directory.*;


public class RoleAuthorizationsTableModel extends AbstractTableModel {
  private static final int _MAX_COLUMNS = 6;
  private static final String[] _columnNames = {"Papel", "Recurso", "Privilgio", "Permisso", "TA", "Status"};
  private static final ImageIcon authoOn  = new ImageIcon(maca_ad.RoleAuthorizationsTableModel.class.getResource("green-on.gif"));
  private static final ImageIcon authoOff = new ImageIcon(maca_ad.RoleAuthorizationsTableModel.class.getResource("red-on.gif"));

  private Role     _role = null;
  private Vector _authos = new Vector();
  private Vector childAuthos  = new Vector();
  private Vector parentAuthos = new Vector();
  private JTable _table  = null;
  private int[] _colsWidth = {0, 0, 0, 0, 0, 0};
  private Color lightGreen = new Color(0, 50, 0, 50);

    class RowRenderer extends DefaultTableCellRenderer {
	public Component getTableCellRendererComponent(JTable table,
                                                       Object value, boolean isSelected,
                                                       boolean hasFocus,
                                                       int row, int column) {
          int realRow = (_table.getModel() instanceof TableSorter)?
                         ((TableSorter)_table.getModel()).indexes[row] : row;
          if(row % 2 == 0) {
            this.setBackground(lightGreen);
          }
          else {
            setBackground(Color.white);
          }
          if (column == 3) {
            if (((row >= 0) && getValueAt(realRow, column).equals("+"))) {
              this.setToolTipText("CONCEDIDA");
            }
            else if (((row >= 0)  && getValueAt(realRow, column).equals("-"))) {
              this.setToolTipText("NEGADA");
            }
            else if (row >= 0) {
              Authorization autho = (Authorization) _authos.get(realRow);
              try {
                this.setToolTipText(autho.getPrivilegeType());
              }
              catch (NamingException exc) {
                this.setToolTipText(exc.getExplanation());
              }
            }
          }
          if ((row >= 0) && (column == 5)) {
            this.setIcon((ImageIcon)getValueAt(realRow, column));
            Authorization autho = (Authorization) _authos.get(realRow);
            try {
              if (autho.getStatus() == autho.ACTIVE) {
                this.setToolTipText("ATIVADA");
              }
              else {
                this.setToolTipText("DESATIVADA");
              }
            }
            catch (NamingException exc) {
                this.setToolTipText(exc.getExplanation());
            }
          }
          if (column == 4) {
            this.setToolTipText("Tipo Autorizao");
          }
          return super.getTableCellRendererComponent(table,
                                                      value, isSelected, hasFocus,
                                                      row, column);
	}
    }

  public void clear() {
    this._authos.clear();
    this.parentAuthos.clear();
    this.childAuthos.clear();
    this.fireTableDataChanged();
  }

  public void filterAutorizations(boolean strong, boolean weak, boolean parent, boolean child,
                                  boolean positive, boolean negative) throws NamingException {
    this._authos.clear();
    if (child) {
      this._authos.addAll(this.childAuthos);
    }
    if (parent) {
      this._authos.addAll(this.parentAuthos);
    }
    next: for (int n = (this._authos.size() - 1); n >= 0; n--) {
      Authorization autho = (Authorization) this._authos.get(n);
      if (!strong && autho.isStrong()) {
        this._authos.remove(n);
        continue next;
      }
      if (!weak && autho.isWeak()) {
        this._authos.remove(n);
        continue next;
      }
      if (!positive && autho.isPositive()) {
        this._authos.remove(n);
        continue next;
      }
      if (!negative && autho.isNegative()) {
        this._authos.remove(n);
        continue next;
      }
    }
    this.adjustColumnsWidth();
    this.fireTableDataChanged();
  }

  public void filterAutorizations(boolean strong, boolean weak, boolean parent, boolean child,
                                  boolean positive, boolean negative,
                                  String resFilter) throws NamingException {
    FontMetrics fm =  this._table.getFontMetrics(this._table.getFont());
    for (int n = 0; n < this._colsWidth.length; n++) this._colsWidth[n] = 0;
    if (_role != null) {
      resFilter = "*" + MACA_Applet.revertResourceName(resFilter);
      this._role.setRoleAuthorizationStatus(parentAuthos, childAuthos, resFilter);
      Enumeration enum = null;
      this._colsWidth[0] = Math.max(this._colsWidth[0], fm.stringWidth(this._role.getName()));
      this._colsWidth[3] = Math.max(fm.stringWidth(this._columnNames[3]), fm.stringWidth("++++"));
      this._colsWidth[4] = Math.max(fm.stringWidth("Fraca"), fm.stringWidth("Forte"));
      this._colsWidth[5] = Math.max(authoOn.getIconWidth(), authoOff.getIconWidth());
      this._table.setRowHeight(Math.max(authoOn.getIconHeight(), authoOff.getIconHeight()));

      for (int n = 0; n < parentAuthos.size(); n++) {
        Authorization autho = (Authorization) parentAuthos.get(n);
        this._colsWidth[0] = Math.max(this._colsWidth[0], fm.stringWidth(autho.getRoleName()));
        this._colsWidth[1] = Math.max(this._colsWidth[1], fm.stringWidth(autho.ldapResourceDNToResourceName(true)));
        this._colsWidth[2] = Math.max(this._colsWidth[2], fm.stringWidth(autho.getPrivilege()));
      }
      for (int n = 0; n < childAuthos.size(); n++) {
        Authorization autho = (Authorization) childAuthos.get(n);
        this._colsWidth[1] = Math.max(this._colsWidth[1], fm.stringWidth(autho.ldapResourceDNToResourceName(true)));
        this._colsWidth[2] = Math.max(this._colsWidth[2], fm.stringWidth(autho.getPrivilege()));
      }
      this.filterAutorizations(strong, weak, parent, child, positive, negative);
    }
  }

  public RoleAuthorizationsTableModel(Role aRole, JTable aTable) throws NamingException {
    this._role   = aRole;
    this._table  = aTable;
    this.authoOff.setDescription("");
    this.authoOn.setDescription("");
    this._table.setModel(new TableSorter(this));
    ((TableSorter)this._table.getModel()).addMouseListenerToHeaderInTable(this._table);
    for (int n = 0; n < this.getColumnCount(); n++) {
      TableColumn col = this._table.getColumn(this._table.getColumnName(n));
      RowRenderer rowRender = new RowRenderer();
      col.setCellRenderer(rowRender);
      if ((n == 2) || (n == 3) || (n == 4) || (n == 5)) {
         rowRender.setHorizontalAlignment(SwingConstants.CENTER);
      }
    }
    ((TableSorter)this._table.getModel()).sortByColumn(1, true);
  }

  public int getColumnCount() {
    return _MAX_COLUMNS;
  }

  public Class getColumnClass(int c) {
    return getValueAt(0, c).getClass();
  }

  public String getColumnName(int col) {
    return _columnNames[col];
  }

  private void adjustColumnsWidth() {
      for (int n = 0; n < this.getColumnCount(); n++) {
        TableColumn column = this._table.getColumn(this._table.getColumnName(n));
        column.setMinWidth(Math.round((float)1.1*(float)this._colsWidth[n]));
      }
      this._table.sizeColumnsToFit(0);
  }


  public Object getValueAt(int row, int col) {
    try {
      Authorization autho = (Authorization) _authos.get(row);
      switch (col) {
        case 0 : return autho.getRoleName();                   // Papeis
        case 1 : return autho.ldapResourceDNToResourceName(true); // Recurso
        case 2 : return autho.getPrivilege();                  // Privilgios
        case 3 : return (autho.isPositive()?"+":(autho.isNegative()?"-":"Regra"));          // Tipo privilgios
        case 4 : return (autho.isStrong()?"Forte":"Fraca");    // Tipo autorizao
        case 5 : return (autho.getStatus().equalsIgnoreCase(Authorization.ACTIVE)?authoOn:authoOff); // Tipo autorizao
      }
    }
    catch (Exception exc) {
      System.out.println(exc);
    }
    return "";
  }

  public Authorization getAutorization(int row) {
    int realRow = (_table.getModel() instanceof TableSorter)?
                           ((TableSorter)_table.getModel()).indexes[row] : row;
    return (Authorization) _authos.get(realRow);
  }

  public int getRowCount() {
    return _authos.size();
  }

  public Role getRole() {
    return this._role;
  }

  public boolean isAuthorizationEditable(Authorization autho) throws NamingException {
    return ((autho != null) && (autho.getRoleDN().equalsIgnoreCase(this._role.getGroupDN())));
  }

  public void removeRow(int row) throws Exception {
    int realRow = (_table.getModel() instanceof TableSorter)?
                           ((TableSorter)_table.getModel()).indexes[row] : row;
    Authorization autho = (Authorization) _authos.get(realRow);
    if (this.isAuthorizationEditable(autho)) {
      this._authos.remove(realRow);
      autho.removeAuthorization();
      this.fireTableRowsDeleted(row, row);
    }
    else {
      throw new Exception("Proibido remover autorizaes herdadas.");
    }
  }

  public void removeRows(int[] rows) throws Exception {
    for (int n = rows.length - 1; n >= 0 ; n--) {
      this.removeRow(rows[n]);
    }
  }

}