package maca_bean;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.http.*;
import javax.swing.*;

public class MacaBean implements javax.servlet.http.HttpSessionBindingListener {

  private static final int PRIVILEGE_ATTRIBUTE_CREDENTIALS = 10;
  private static org.omg.CORBA.ORB orb = null;

  private String userID = "";
  private byte[] password = null;
  private byte[] newPassword = null;
  private byte[] newPasswordAgain = null;
  private String selectedRole = "";
  private boolean firstLogin = false;
  private String  authMessage = "";
  private org.omg.maca_cs.SecurityLevel2.Credentials creds = null;
  private org.omg.maca_cs.DfResourceAccessDecision.AccessDecision ado = null;
  private maca_cs.Monitor monitor = null;
  private org.omg.maca_cs.Security.AttributeType[] attrTypeList = null;
  private String maca_cs_URL = "http://127.0.0.1";
  private String paIOR = "", adoIOR = "", monitorIOR = "";

  private void clearPassword(byte[] pwd) {
    for (int i = 0; i < pwd.length; i++) {
      pwd[i] = 0;
    }
  }

  private void maca_cs_URL_Setup() {
    String[] maca_cs_URLs = this.get_maca_cs_URLs();
    if (maca_cs_URLs.length > 0) {
      maca_cs_URL =  maca_cs_URLs[0] + "/maca_cs.iors";
    }
  }

  private void readIORs() throws Exception {
    URL iorURL = new URL(maca_cs_URL);
    BufferedReader iorStream = new BufferedReader(new InputStreamReader(iorURL.openStream()));
    String line = iorStream.readLine();
    StringTokenizer strTk = new StringTokenizer(line, ";");
    while (strTk.hasMoreTokens()) {
      String ior = strTk.nextToken();
      if (ior.startsWith("paIOR")) {
        paIOR = ior.substring("pa".length());
      }
      else if (ior.startsWith("adoIOR")) {
        adoIOR = ior.substring("ado".length());
      }
      else if (ior.startsWith("monitorIOR")) {
        monitorIOR = ior.substring("monitor".length());
      }
    }
    iorStream.close();
  }

  private void RedirectToHTTPS(javax.servlet.jsp.PageContext pageContext)  throws java.io.IOException {
    javax.servlet.http.HttpServletRequest request = (javax.servlet.http.HttpServletRequest) pageContext.getRequest();
    javax.servlet.http.HttpServletResponse response = (javax.servlet.http.HttpServletResponse) pageContext.getResponse();
    if (!request.getScheme().equalsIgnoreCase("https")) {
     response.sendRedirect("https://"+request.getServerName()+request.getRequestURI());
    }
  }

  private boolean isValid() {
    if(creds != null && ado != null && monitor != null){
      try {
        return creds.refresh();
      }
      catch (Exception ex) {
        System.out.println(ex.getMessage());
      }
    }
    return false;
  }

  //Mtodos pblicos

  public MacaBean() {
  }

  public String[] get_maca_cs_URLs() {
    StringTokenizer urls = new StringTokenizer(System.getProperty("maca_bean.maca_cs.url"), ";");
    String[] maca_cs_URLs = new String[urls.countTokens()];
    for (int i = 0; urls.hasMoreTokens(); i++) {
      maca_cs_URLs[i] = urls.nextToken();
    }
    return maca_cs_URLs;
  }

  public void set_maca_cs_URLs(String url) {
    maca_cs_URL = url + "/maca_cs.iors";
  }

  public void initProperties(String propertiesPath) {
      try {
        String[] args = null;
        Properties props = new Properties();
        System.setProperty("maca_bean.maca_cs.url", "http://127.0.0.1");
        System.setProperty("maca_bean.web.session.timeout", "30");
        props.load(new FileInputStream(propertiesPath+"/maca_bean.properties"));
        System.getProperties().putAll(props);
        maca_cs_URL_Setup();

        if (orb == null) {
          Properties orbProps = new Properties();
          orbProps.load(new FileInputStream(propertiesPath+"/orb.properties"));
          System.getProperties().putAll(orbProps);
          orb = org.omg.CORBA.ORB.init(args, orbProps);
        }
        org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
        org.omg.maca_cs.Security.AttributeType[] auxAttrType = {new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value),
                                                      new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.AccessId.value)};
        attrTypeList = auxAttrType;
      }
      catch (java.io.IOException exc) {
        System.out.println(exc);
      }
  }

   //Mtodo chamado quando a sesso inicia-se
   public void valueBound(javax.servlet.http.HttpSessionBindingEvent event) {
   //  System.out.println("sessao iniciada!!!!!!!!!!!!!!!!!111");
   }

   //Mtodo chamado quando a sesso termina
   public void valueUnbound(javax.servlet.http.HttpSessionBindingEvent event) {
    // System.out.println("sessao encerrada!!!!!!!!!!!!!!!!!111");
     this.logout(event.getSession());
   }

  //Access UserID property
  public String getUserID() {
    return userID;
  }

  public String getAuthenticationMessage() {
    return authMessage;
  }

  //Access UserID property
  public void setUserID(String newValue) {
    if (newValue!=null) {
      userID = newValue;
    }
  }

  public String getUserRole() {
    try {
      if (this.isValid()) {
        org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
        org.omg.maca_cs.Security.AttributeType[] attrTypeRoleList = {new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value)};
        org.omg.maca_cs.Security.SecAttribute[] attrs = creds.get_attributes(attrTypeRoleList);
        for (int n = 0; n < attrs.length; n++) {
          if ((attrs[n].attribute_type.attribute_family.family == 1) && // privilege attributes; family = 1
              (attrs[n].attribute_type.attribute_type == org.omg.maca_cs.Security.Role.value)) {
            return new String(attrs[n].value);
          }
        }
      }
    }
    catch (Exception exc) {
      System.out.println(exc);
    }
    return "";
  }

  public String executeRequest(String request) {
    try {
      if (this.isValid()) {
        return monitor.executeRequest(creds, request);
      }
    }
    catch (Exception exc) {
      System.out.println(exc);
    }
    return "";
  }

  public String getDefaultRole() {
    try {
      if (this.isValid()) {
        org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
        org.omg.maca_cs.Security.AttributeType[] attrTypeRoleList = {new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value)};
        org.omg.maca_cs.Security.SecAttribute[] attrs = creds.get_attributes(attrTypeRoleList);
        for (int n = 1; n < attrs.length; n++) {
          if ((attrs[n].attribute_type.attribute_family.family == 1) && // privilege attributes; family = 1
              (attrs[n].attribute_type.attribute_type == org.omg.maca_cs.Security.Role.value)) {
            return new String(attrs[n].value);
          }
        }
      }
    }
    catch (Exception exc) {
      System.out.println(exc.getMessage());
    }
    return "";
  }

  //Retorna as credencias do usurio
  public org.omg.maca_cs.SecurityLevel2.Credentials getCredentials() throws Exception {
    try {
      if (this.isValid()) {
        return creds;
      }
    }
    catch (Exception exc) {
      System.out.println(exc);
      throw exc;
    }
    throw new Exception("Sesso do usurio '"+userID+"' invlida.");
  }

  //Retorna as credencias do usurio
  public org.omg.maca_cs.DfResourceAccessDecision.AccessDecision getAccessDecisionObject() throws Exception {
    try {
      if (this.isValid()) {
        return ado;
      }
    }
    catch (Exception exc) {
      System.out.println(exc);
      throw exc;
    }
    throw new Exception("Sesso do usurio '"+userID+"' invlida.");
  }

  public String[] getUserRoles() {
    String[] rolesList = null;
    try {
      if (this.isValid()) {
        org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
        org.omg.maca_cs.Security.AttributeType[] attrTypeRoleList = {new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value)};
        org.omg.maca_cs.Security.SecAttribute[] attrs = creds.get_attributes(attrTypeRoleList);
        rolesList = new String[attrs.length];
        for (int n = 0; n < attrs.length; n++) {
          if ((attrs[n].attribute_type.attribute_family.family == 1) && // privilege attributes; family = 1
              (attrs[n].attribute_type.attribute_type == org.omg.maca_cs.Security.Role.value)) {
            rolesList[n] = new String(attrs[n].value);
          }
        }
      }
    }
    catch (Exception exc) {
      System.out.println(exc.getMessage());
    }
    return rolesList;
  }

  //Access selectedRole property
  public String getSelectedRole() {
    return selectedRole;
  }

  //Access selectedRole property
  public void setSelectedRole(String newValue) {
    if (newValue!=null) {
      selectedRole = newValue;
    }
  }

  //Access Password property
  public void setPassword(String newValue) {
    if ((newValue!=null) && (!newValue.equalsIgnoreCase(""))) {
      password = newValue.getBytes();
    }
  }

  //Access newPassword property
  public void setNewPassword(String newValue) {
    if ((newValue!=null) && (!newValue.equalsIgnoreCase(""))) {
      newPassword = newValue.getBytes();
    }
  }

  //Access newPasswordAgain property
  public void setNewPasswordAgain(String newValue) {
    if ((newValue!=null) && (!newValue.equalsIgnoreCase(""))) {
      newPasswordAgain = newValue.getBytes();
    }
  }

  public boolean isReady(javax.servlet.jsp.PageContext pageContext)  throws java.io.IOException {
    this.RedirectToHTTPS(pageContext);
    return (!userID.equalsIgnoreCase("")) && (password != null);
  }

  public boolean isReadyToChangePassword(javax.servlet.jsp.PageContext pageContext)  throws java.io.IOException {
    return (this.isReady(pageContext)) &&
           (newPassword != null) &&
           (newPasswordAgain != null);
  }

  public boolean isFirstLogin() {
    return firstLogin;
  }

  public boolean isValid(javax.servlet.jsp.PageContext pageContext)   {
    try {
      return this.isValid() && pageContext.getRequest().getScheme().equalsIgnoreCase("https");
    }
    catch (Exception exc) {
      System.out.println(exc.getMessage());
    }
    return false;
  }

  public synchronized String changePassword(javax.servlet.http.HttpSession session) {
    String result = "Obrigatrio o preenchimento de todos os campos!";
    try {
      if ((this.isValid()) && (password != null) && (newPassword != null) && (newPasswordAgain != null)) {
        String passwordStr = new String(password);
        String newPasswordStr = new String(newPassword);
        String newPasswordAgainStr = new String(newPasswordAgain);
        if ((!newPasswordStr.equalsIgnoreCase("")) &&
            (newPasswordStr.equals(newPasswordAgainStr))) {
          if (!passwordStr.equals(newPasswordStr)) {
            if (!userID.equalsIgnoreCase(newPasswordStr)) {
              org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
              org.omg.maca_cs.Security.AttributeType    attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, PRIVILEGE_ATTRIBUTE_CREDENTIALS);
              org.omg.maca_cs.Security.SecAttribute[] reqstedPriv = {new org.omg.maca_cs.Security.SecAttribute(attrType, new byte [0], (passwordStr+"\n"+newPasswordStr).getBytes())};
              org.omg.maca_cs.Security.AttributeListHolder actualPriv = new org.omg.maca_cs.Security.AttributeListHolder();
              if (creds.set_privileges(true, reqstedPriv, actualPriv)) {
                if (firstLogin) {
                  this.setActiveRole(this.getDefaultRole());
                  firstLogin = false;
                }
                result = "Alteraao de senha efetuada com sucesso!";
              }
              else {
                for (int n = 0; n < actualPriv.value.length; n++) {
                  if ((actualPriv.value[n].attribute_type.attribute_family.family_definer == 0) &&
                      (actualPriv.value[n].attribute_type.attribute_family.family == 1) &&
                      (actualPriv.value[n].attribute_type.attribute_type == PRIVILEGE_ATTRIBUTE_CREDENTIALS)) {
                    result = new String(actualPriv.value[n].value);
                  }
                }
              }
            }
            else {
              result = "Nova senha deve ser diferente do nome de login do usurio!";
            }
          }
          else {
            result = "Nova senha deve ser diferente da atual!";
          }
        }
        else if (newPassword.length > 0) {
          result = "Nova senha e confirmao da nova senha devem ser iguais!";
        }
        else {
          result = "Nova senha no pode ficar em branco!";
        }
      }
      this.clearPassword(password);
      password = null;
      this.clearPassword(newPassword);
      newPassword = null;
      this.clearPassword(newPasswordAgain);
      newPasswordAgain = null;
    }
    catch (Exception exc) {
      System.out.println(exc.toString());
    }
    return result;
  }

  public boolean setActiveRole(String newRole) {
    try {
      if (this.isValid()) {
        org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
        org.omg.maca_cs.Security.AttributeType    attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value);
        org.omg.maca_cs.Security.SecAttribute[] reqstedPriv = {new org.omg.maca_cs.Security.SecAttribute(attrType, new byte [0], newRole.getBytes())};
        org.omg.maca_cs.Security.AttributeListHolder actualPriv = new org.omg.maca_cs.Security.AttributeListHolder();
        return creds.set_privileges(true, reqstedPriv, actualPriv);
      }
      else {
        return false;
      }
    }
    catch (Exception exc) {
      System.out.println(exc.toString());
    }
    return false;
  }

  public boolean access_allowed (String resource_name, String operation) {
    if ((creds != null) && (ado != null)) {
      org.omg.maca_cs.Security.SecAttribute[] attrs = creds.get_attributes(attrTypeList);
      org.omg.maca_cs.DfResourceAccessDecision.ResourceNameComponent[] resNameCompts = {new org.omg.maca_cs.DfResourceAccessDecision.ResourceNameComponent("Aplicacoes", resource_name)};
      org.omg.maca_cs.DfResourceAccessDecision.ResourceName resName = new org.omg.maca_cs.DfResourceAccessDecision.ResourceName("ldap.incor.usp.br", resNameCompts);
      try {
        return ado.access_allowed(resName,
                                  operation.equalsIgnoreCase("Execucao") ? "Execuo" :
                                                                            operation,
                                  attrs);
      }
      catch (Exception exc) {
        System.out.println(exc.getMessage());
      }
    }
    return false;
  }

  public synchronized void logout(javax.servlet.http.HttpSession session) {
    if ((creds != null) /*&& creds._is_bound()*/) {
      try {
        creds.destroy();
      }
      catch (Exception exc) {
        System.out.println(exc.getMessage());
      }
      creds = null;
    }
    try {
      if ((session != null) && !session.isNew()) {
        session.invalidate();
      }
    }
    catch (java.lang.IllegalStateException exc) {
    //exc.printStackTrace();
    }
    finally {
     userID = "";
     creds = null;
     password = null;
     ado = null;
    }
  }

  public boolean authenticate(javax.servlet.http.HttpSession session) {
    try {
      org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
      org.omg.maca_cs.Security.AttributeType     attrType = new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value);
      org.omg.maca_cs.Security.SecAttribute[]       attrs = {new org.omg.maca_cs.Security.SecAttribute(attrType, new byte [0], "".getBytes())};
      org.omg.maca_cs.SecurityLevel2.CredentialsHolder credHolder = new org.omg.maca_cs.SecurityLevel2.CredentialsHolder();
      org.omg.maca_cs.Security.OpaqueHolder  contData = new org.omg.maca_cs.Security.OpaqueHolder();
      org.omg.maca_cs.Security.OpaqueHolder authEspec = new org.omg.maca_cs.Security.OpaqueHolder();
      System.out.println("maca_bean.maca_cs.url: "+maca_cs_URL);
      this.readIORs();
      firstLogin  = false;
      org.omg.CORBA.Object corbaObj = orb.string_to_object(paIOR);
      org.omg.maca_cs.SecurityLevel2.PrincipalAuthenticator pa = org.omg.maca_cs.SecurityLevel2.PrincipalAuthenticatorHelper.narrow(corbaObj);
      org.omg.maca_cs.Security.AuthenticationStatus authStatus = pa.authenticate(0, userID, password, attrs, credHolder, contData, authEspec);
      if (authStatus.value() == org.omg.maca_cs.Security.AuthenticationStatus.SecAuthSuccess.value() ||
          authStatus.value() == org.omg.maca_cs.Security.AuthenticationStatus.SecAuthContinue.value()) {
        creds = credHolder.value;
        corbaObj = orb.string_to_object(adoIOR);
        ado = org.omg.maca_cs.DfResourceAccessDecision.AccessDecisionHelper.narrow(corbaObj);
        corbaObj = orb.string_to_object(monitorIOR);
        monitor = maca_cs.MonitorHelper.narrow(corbaObj);
        session.setMaxInactiveInterval(Integer.parseInt(System.getProperty("maca_bean.web.session.timeout")) * 60);
        firstLogin = (authStatus.value() == org.omg.maca_cs.Security.AuthenticationStatus.SecAuthContinue.value());
        this.clearPassword(password);
        password = null;
        if (!firstLogin) this.setActiveRole(this.getDefaultRole());
        return true;
      }
      authMessage = new String(authEspec.value);
      try {
        session.invalidate();
      }
      catch (java.lang.IllegalStateException exc) {
        //exc.printStackTrace();
      }
    }
    catch (java.net.MalformedURLException exc) {
      System.out.println(exc.getMessage());
    }
    catch (Exception exc) {
      System.out.println(exc.getMessage());
    }
    return false;
  }

}