package maca_cs;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import maca_ca.*;
import java.util.*;
import org.omg.PortableServer.*;

public class AccessDecision implements org.omg.maca_cs.DfResourceAccessDecision.AccessDecisionOperations {

  private POA _poa = null;
  private org.omg.CORBA.ORB _orb = null;
  private LogWriter         _log = new LogWriter(new PrintWriter(System.out, true));

  private String[] extractResourceParameters(String resource) {
    String[] strs = new String[2];
    int pos = resource.indexOf("(");
    if (pos == -1) {
      strs[0] = resource;
      strs[1] = "";
    }
    else {
      strs[0] = resource.substring(0, pos);
      int lastPos = resource.lastIndexOf(")");
      if (lastPos != -1) {
        strs[1] = resource.substring(pos, lastPos+1);
      }
      else {
        strs[1] = resource.substring(pos, resource.length()) + ")";
      }
    }
    return strs;
  }

  public AccessDecision(LogWriter aLog, org.omg.CORBA.ORB anORB, POA aPoa) {
    _poa = aPoa;
    _orb = anORB;
    if (aLog != null) {
      _log = aLog;
    }
  }

  public boolean access_allowed (org.omg.maca_cs.DfResourceAccessDecision.ResourceName resource_name,
                                 java.lang.String operation,
                                 org.omg.maca_cs.Security.SecAttribute[] attribute_list) throws org.omg.maca_cs.DfResourceAccessDecision.InternalError {
    if (attribute_list.length > 0) {
      String userName = "";
      String activeRoles = "";
      for (int n = 0; n < attribute_list.length; n++) {
        if ((attribute_list[n].attribute_type.attribute_family.family_definer == 0) && // OMG famlily
            (attribute_list[n].attribute_type.attribute_family.family == 1) && // Privilege attributes family
            (attribute_list[n].attribute_type.attribute_type == org.omg.maca_cs.Security.AccessId.value)) { // login_name
          userName = new String(attribute_list[n].value).toLowerCase().trim();
          break;
        }
      }
      try {
        if (!userName.equalsIgnoreCase("")) {
          UserHolder _userHolder = (UserHolder) AuthenticatorServer.users.get(userName);
          if (_userHolder != null) {
          	if (!_userHolder.isLocked()) {
              User _user = _userHolder.getUser();
              activeRoles = _user.getActiveRolesName();
              if (resource_name.resource_name_component_list.length > 0) {
                String[] strs = this.extractResourceParameters(resource_name.resource_name_component_list[0].value_string);
                boolean result = _user.hasAccess(strs[0]+";"+resource_name.resource_name_component_list[0].name_string, strs[1], operation);
                _log.println(java.util.logging.Level.INFO, userName+" :: "+activeRoles,
                             "maca_cs.AccessDecision", "access_allowed",
                             "Acesso "+(result ? "autorizado":"no autorizado")+": "+resource_name.resource_name_component_list[0].value_string+" :: "+operation);
                if (result) {
                  Statistics.newSuccessfulAuthorization();
                }
                else {
                  Statistics.newUnsuccessfulAuthorization();
                }
                return result;
              }
              else {
                _log.println(java.util.logging.Level.WARNING, userName+" :: "+activeRoles,
                             "maca_cs.AccessDecision", "access_allowed",
                             "Nome do recurso no fornecido");
                Statistics.newUnsuccessfulAuthorization();
                return false;
              }
            }
            else {
              _log.println(java.util.logging.Level.WARNING, userName+" :: "+activeRoles,
                           "maca_cs.AccessDecision", "access_allowed",
                           "Acesso no autorizado enquando no mudar a senha");
              Statistics.newUnsuccessfulAuthorization();
              return false;
            }
          }
          else {
            _log.println(java.util.logging.Level.WARNING, userName+" :: "+activeRoles,
                         "maca_cs.AccessDecision", "access_allowed",
                         "Usurio no possui sesses abertas");
            Statistics.newUnsuccessfulAuthorization();
            return false;
          }
        }
        else {
          _log.println(java.util.logging.Level.WARNING, "maca_cs",
                       "maca_cs.AccessDecision", "access_allowed",
                       "Nome do usurio no fornecido");
          Statistics.newUnsuccessfulAuthorization();
          return false;
        }
      }
      catch (java.lang.NullPointerException exc) {
        Statistics.newUnsuccessfulAuthorization();
        _log.println(java.util.logging.Level.SEVERE, userName+" :: "+activeRoles,
                     "maca_cs.AccessDecision", "access_allowed",
                     exc.getMessage());
      }
      catch (Exception exc) {
        _log.println(java.util.logging.Level.SEVERE, userName+" :: "+activeRoles,
                     "maca_cs.AccessDecision", "access_allowed",
                     exc.getMessage());
        System.out.println("access_allowed "+exc);
        Statistics.newUnsuccessfulAuthorization();
        throw new org.omg.maca_cs.DfResourceAccessDecision.InternalError(exc.toString(), org.omg.maca_cs.DfResourceAccessDecision.InternalErrorType.NotFatal);
      }
    }
    _log.println(java.util.logging.Level.SEVERE, "maca_cs",
                 "maca_cs.AccessDecision", "access_allowed",
                 "Atributos de segurana no fornecidos");
    Statistics.newUnsuccessfulAuthorization();
    return false;
  }


  public boolean[] multiple_access_allowed (org.omg.maca_cs.DfResourceAccessDecision.AccessDefinition[] access_requests,
                                                         org.omg.maca_cs.Security.SecAttribute[] attribute_list) throws org.omg.maca_cs.DfResourceAccessDecision.InternalError {
    boolean[] results = new boolean[access_requests.length];
    if ((access_requests.length > 0) && (attribute_list.length > 0)) {
      String userName = "";
      String activeRoles = "";
      for (int n = 0; n < attribute_list.length; n++) {
        if ((attribute_list[n].attribute_type.attribute_family.family_definer == 0) && // OMG famlily
            (attribute_list[n].attribute_type.attribute_family.family == 1) && // Privilege attributes family
            (attribute_list[n].attribute_type.attribute_type == org.omg.maca_cs.Security.AccessId.value)) { // login_name
          userName = new String(attribute_list[n].value).toLowerCase().trim();
          break;
        }
      }
      try {
        if (!userName.equalsIgnoreCase("")) {
          UserHolder _userHolder = (UserHolder) AuthenticatorServer.users.get(userName);
          if (_userHolder != null) {
          	if (!_userHolder.isLocked()) {
              User _user = _userHolder.getUser();
              activeRoles = _user.getActiveRolesName();
              for (int n = 0; n < access_requests.length; n++) {
                org.omg.maca_cs.DfResourceAccessDecision.AccessDefinition accessDef = access_requests[n];
                if (accessDef.resource_name.resource_name_component_list.length > 0) {
                  String[] strs = this.extractResourceParameters(accessDef.resource_name.resource_name_component_list[0].value_string);
                  results[n] = _user.hasAccess(strs[0]+";"+accessDef.resource_name.resource_name_component_list[0].name_string, strs[1], accessDef.operation);
                  _log.println(java.util.logging.Level.INFO, userName+" :: "+activeRoles,
                               "maca_cs.AccessDecision", "multiple_access_allowed",
                               "Acesso "+(results[n] ? "autorizado":"no autorizado")+": "+accessDef.resource_name.resource_name_component_list[0].value_string+" :: "+accessDef.operation);
                  if (results[n]) {
                    Statistics.newSuccessfulAuthorization();
                  }
                  else {
                    Statistics.newUnsuccessfulAuthorization();
                  }
                }
                else {
                  results[n] = false;
                  Statistics.newUnsuccessfulAuthorization();
                  _log.println(java.util.logging.Level.WARNING, userName+" :: "+activeRoles,
                               "maca_cs.AccessDecision", "multiple_access_allowed",
                               "Nome do recurso no fornecido");
                }
              }
          	}
            else {
              Statistics.newUnsuccessfulAuthorization();
              _log.println(java.util.logging.Level.WARNING, userName+" :: "+activeRoles,
                           "maca_cs.AccessDecision", "access_allowed",
                           "Acesso no autorizado enquando no mudar a senha");
            }
          }
          else {
            Statistics.newUnsuccessfulAuthorization();
            _log.println(java.util.logging.Level.WARNING, userName+" :: "+activeRoles,
                         "maca_cs.AccessDecision", "multiple_access_allowed",
                         "Nome do recurso ou operao (modo de acesso) no fornecidos");
          }
        }
        else {
          Statistics.newUnsuccessfulAuthorization();
          _log.println(java.util.logging.Level.WARNING, "maca_cs",
                       "maca_cs.AccessDecision", "multiple_access_allowed",
                       "Nome do usurio no fornecido");
        }
      }
      catch (Exception exc) {
        _log.println(java.util.logging.Level.SEVERE, userName+" :: "+activeRoles,
                     "maca_cs.AccessDecision", "multiple_access_allowed",
                     exc.getMessage());
        System.out.println("multiple_access_allowed "+exc);
        Statistics.newUnsuccessfulAuthorization();
        throw new org.omg.maca_cs.DfResourceAccessDecision.InternalError(exc.toString(), org.omg.maca_cs.DfResourceAccessDecision.InternalErrorType.NotFatal);
      }
    }
    return results;
  }

}