package maca_cs;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2005</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.2
 */

import java.io.*;
import maca_ca.*;
import java.util.*;
import java.security.*;
import org.omg.CosNaming.*;
import org.omg.PortableServer.*;
import org.omg.CosNaming.NamingContextExtPackage.*;

public class AuthenticatorServer implements Stopable {

  //Variveis de classe privadas
  private static java.util.Timer maca_cs_timer = new java.util.Timer(true);

  //PROPRIEDADES
  private static String IOR_OUTPUT_DIR         = "";

  //Variveis de classe protegidas
  protected static org.omg.CORBA.ORB         orb = null;
  protected static Hashtable            contexts = new Hashtable();
  protected static Hashtable               users = new Hashtable(1000);
  protected static LogWriter                 log = new LogWriter(new PrintWriter(System.out, true));
  protected static LogWriter            statsLog = new LogWriter(new PrintWriter(System.out, true));

  //Objetos servidores disponibilizados para interno ao MACA CS
  //ADO - Access Decision Object
  protected static org.omg.maca_cs.DfResourceAccessDecision.AccessDecision ado = null;
  //PA - Principal Authenticator
  protected static org.omg.maca_cs.SecurityLevel2.PrincipalAuthenticator    pa = null;
  //Monitor do MACA CS
  protected static Monitor                                     monitor = null;


  //PROPRIEDADES
  protected static String[] USER_ACCOUNT_ATTR_LIST = new String[0];
  protected static String          SERVER_AND_PORT = "";
  protected static String                     ROOT = "";
  protected static boolean                  IS_SSL = false;
  protected static int                CRED_TIMEOUT = 30 * 60 * 1000; // 30 minutos em milissegundos (default)
                                                                     // Tempo de timeout para objetos Credentials
  protected static int                   MAX_USERS = 0;            // Nmero mximo de usurios simultneos

  class GarbageColectorTimer extends java.util.TimerTask {
    public void run() {
      //System.out.println(" >>> "+maca_cs.AuthenticatorServer.users.size());
      System.gc();
    }
  }

  //Mtodos privados
  private static void loadProperties() throws Exception {
    maca_ca.LoadProperties.init(null);
    maca_ca.UserSchema.init();
    System.setProperty("maca_cs.log.StatisticsRefreshTime", "5"); //Valor default: 60 minutos
    System.setProperty("maca_cs.log.Path", "");                    //Valor default: sada padro
    System.setProperty("maca_cs.LoginTimeout", "60");              //Valor default: 60 minutos
    System.setProperty("maca_cs.ClientCallbackRefreshTime", "5");  //Valor default: 05 minutos
    System.setProperty("maca_cs.IOROutputDir", ".");               //Valor default: diretrio corrente
    System.setProperty("maca_cs.ldap.userAccountAttrList", "");    //Valor default: nenhum atributo
    System.setProperty("maca_cs.contexts", "dtCtx,contexts.DateTimeContext"); //Valor default: contexto de datas
    System.setProperty("maca_cs.service.stop.port", "8375");       //Valor default: 8375
    System.setProperty("maca_cs.user.cache", "false");             //Valor default: false
    System.setProperty("maca_cs.user.maxNumber", "0");           //Valor defalt: 0 - sem limite de usurios simutneos
    System.setProperty("maca_cs.user.maxSessionsPerUser", "0");  //Valor defalt: 0 - sem limite para o nmero mximo de sesses simultneas por usurio
    System.setProperty("maca_cs.naming.useService", "false");
    System.setProperty("maca_cs.naming.rootContext", "maca_cs.ctx");
    System.setProperty("maca_cs.naming.authenticatorsSubContext", "authenticators.ctx");
    System.setProperty("maca_cs.naming.authorizersSubContext", "authorizers.ctx");
    System.setProperty("maca_cs.naming.adminSubContext", "admin.ctx");
    System.setProperty("maca_cs.naming.PrincipalAuthenticator", "pa.server");
    System.setProperty("maca_cs.naming.AccessDecisionObject", "ado.server");
    System.setProperty("maca_cs.naming.Monitor", "monitor.server");

    SERVER_AND_PORT = System.getProperty("maca_ca.ldap.server");
    System.out.println("LDAP server: "+SERVER_AND_PORT);
    ROOT            = System.getProperty("maca_ca.ldap.peopleRDN")+","+System.getProperty("maca_ca.ldap.rootDN");
    System.out.println("Users root : "+ROOT);

    if (!System.getProperty("maca_ca.ldap.ssl").equalsIgnoreCase("false")) {
      IS_SSL = true;
      System.out.println("LDAP SSL: true");
    }

    InputStream st = AuthenticatorServer.class.getResourceAsStream("/maca_cs.properties");
    if (st != null) {
      System.getProperties().load(st);
      st.close();
      System.out.println("Recurso /maca_cs.properties carregado");
    }
    try {
      st = new FileInputStream("./maca_cs.properties");
      Properties fileProps = new Properties();
      fileProps.load(st);
      System.getProperties().putAll(fileProps);
      st.close();
      System.out.println("Arquivo ./maca_cs.properties carregado");
    }
    catch (java.io.FileNotFoundException exc) {
    }

    CRED_TIMEOUT = Integer.parseInt(System.getProperty("maca_cs.LoginTimeout")) * 60 * 1000;
    MAX_USERS    = Integer.parseInt(System.getProperty("maca_cs.user.maxNumber"));

    String logPath = System.getProperty("maca_cs.log.Path");
    if (!logPath.equalsIgnoreCase("")) {
      logPath = (!logPath.equalsIgnoreCase("") && logPath.endsWith("/"))?logPath:logPath+"/";
      log      = new LogWriter(logPath+"maca_cs.log");
      statsLog = new LogWriter(logPath+"maca_cs.stats.log");
    }

    IOR_OUTPUT_DIR = System.getProperty("maca_cs.IOROutputDir");
    IOR_OUTPUT_DIR = (!IOR_OUTPUT_DIR.equalsIgnoreCase("") && IOR_OUTPUT_DIR.endsWith("/"))?
                      IOR_OUTPUT_DIR:
                      IOR_OUTPUT_DIR+"/";

    String userAccountAttrListStr = System.getProperty("maca_cs.ldap.userAccountAttrList");
    if (!userAccountAttrListStr.equalsIgnoreCase("")) {
      StringTokenizer strTk = new StringTokenizer(userAccountAttrListStr, ",");
      USER_ACCOUNT_ATTR_LIST = new String[strTk.countTokens()];
      for (int n = 0; strTk.hasMoreTokens(); n++) {
        USER_ACCOUNT_ATTR_LIST[n] = strTk.nextToken().trim();
      }
    }

    st = AuthenticatorServer.class.getResourceAsStream("/orb.properties");
    if (st != null) {
      System.getProperties().load(st);
      st.close();
      System.out.println("Recurso /orb.properties carregado");
    }
    try {
      st = new FileInputStream("./orb.properties");
      Properties fileProps = new Properties();
      fileProps.load(st);
      System.getProperties().putAll(fileProps);
      st.close();
      System.out.println("Arquivo ./orb.properties carregado");
    }
    catch (java.io.FileNotFoundException exc) {
    }
  }

  private static synchronized void loadContexts() {
    String contextsStr = System.getProperty("maca_cs.contexts");
    StringTokenizer ctxs = new StringTokenizer(contextsStr, ";");
    String ctxName  = "";
    String ctxClass = "";
    while (ctxs.hasMoreTokens()) {
      try {
        StringTokenizer ctx = new StringTokenizer(ctxs.nextToken(), ",");
        ctxName  = ctx.nextToken();
        ctxClass = ctx.nextToken();
        if (!contexts.containsKey(ctxName)) {
          contexts.put(ctxName, Class.forName(ctxClass).newInstance());
          System.out.println("Contexto "+ctxName+"["+ctxClass+"] carregado.");
          log.println(java.util.logging.Level.INFO, "maca_cs",
                      "maca_cs.AuthenticatorServer", "loadContexts",
                      "Contexto "+ctxName+" - "+ctxClass+" - carregado");
        }
      }
      catch (Exception exc) {
        System.out.println("Falha carregando contexto "+ctxName+"["+ctxClass+"]: "+exc.toString());
        log.println(java.util.logging.Level.WARNING, "maca_cs",
                    "maca_cs.AuthenticatorServer", "loadContexts",
                    "Falha carregando contexto "+ctxName+" - "+ctxClass+": "+exc.toString());
      }
    }
  }

  private POA createPrincipalAuthenticatorPOA(POA rootPOA) throws org.omg.PortableServer.POAPackage.InvalidPolicy,
                                                                  org.omg.PortableServer.POAPackage.AdapterAlreadyExists,
                                                                  org.omg.CORBA.PolicyError {
    org.omg.CORBA.Policy[] policies = {
      rootPOA.create_lifespan_policy(LifespanPolicyValue.TRANSIENT),
      rootPOA.create_servant_retention_policy(ServantRetentionPolicyValue.RETAIN),
      rootPOA.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID),
      rootPOA.create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID),
    };
    POA pa_POA = rootPOA.create_POA("pa_POA", rootPOA.the_POAManager(), policies);
    return pa_POA;
  }

  private POA createAccessDecisionPOA(POA rootPOA) throws org.omg.PortableServer.POAPackage.InvalidPolicy,
                                                          org.omg.PortableServer.POAPackage.AdapterAlreadyExists,
                                                          org.omg.CORBA.PolicyError {

    org.omg.CORBA.Policy[] policies = {
      rootPOA.create_lifespan_policy(LifespanPolicyValue.TRANSIENT),
      rootPOA.create_servant_retention_policy(ServantRetentionPolicyValue.RETAIN),
      rootPOA.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID),
      rootPOA.create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID),
    };
    POA ado_POA = rootPOA.create_POA("ado_POA", rootPOA.the_POAManager(), policies);
    return ado_POA;
  }

  private POA createMonitorPOA(POA rootPOA) throws org.omg.PortableServer.POAPackage.InvalidPolicy,
                                                   org.omg.PortableServer.POAPackage.AdapterAlreadyExists,
                                                   org.omg.CORBA.PolicyError {

    org.omg.CORBA.Policy[] policies = {
      rootPOA.create_lifespan_policy(LifespanPolicyValue.TRANSIENT),
      rootPOA.create_servant_retention_policy(ServantRetentionPolicyValue.RETAIN),
      rootPOA.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID),
      rootPOA.create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID),
    };
    POA monitor_POA = rootPOA.create_POA("monitor_POA", rootPOA.the_POAManager(), policies);
    return monitor_POA;
  }

  private POA createCredentialsPOA(POA rootPOA) throws org.omg.PortableServer.POAPackage.InvalidPolicy,
                                                 org.omg.PortableServer.POAPackage.AdapterAlreadyExists,
                                                 org.omg.CORBA.PolicyError {
    org.omg.CORBA.Policy[] policies = {
         rootPOA.create_thread_policy(ThreadPolicyValue.ORB_CTRL_MODEL),
         rootPOA.create_lifespan_policy(LifespanPolicyValue.TRANSIENT),
         rootPOA.create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID),
         rootPOA.create_id_assignment_policy(IdAssignmentPolicyValue.SYSTEM_ID),
         rootPOA.create_servant_retention_policy(ServantRetentionPolicyValue.RETAIN),
         rootPOA.create_request_processing_policy(RequestProcessingPolicyValue.USE_ACTIVE_OBJECT_MAP_ONLY),
         rootPOA.create_implicit_activation_policy(ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION)
    };
    POA creds_POA = rootPOA.create_POA("creds_POA", rootPOA.the_POAManager(), policies);
    return creds_POA;
  }


  //Mtodos pblicos
  static public UserHolder getUserHolder(String uid) {
    return (UserHolder) users.get(uid);
  }


  public NamingContextExt getRootNamingContext(LogWriter aLog, org.omg.CORBA.ORB anORB) {
    NamingContextExt rootContext = null, subContext = null;
    if ("true".equalsIgnoreCase("maca_cs.naming.useService")) {
    	try {
    		rootContext = NamingContextExtHelper.narrow(anORB.resolve_initial_references("NameService"));
    		try {
    			subContext =
    				NamingContextExtHelper.narrow(rootContext.resolve_str(System.getProperty("maca_cs.naming.rootContext")));
    		}
    		catch (org.omg.CosNaming.NamingContextPackage.NotFound exc) {
    			rootContext.bind_new_context(rootContext.to_name(System.getProperty("maca_cs.naming.rootContext")));
    		}
    		try {
    			subContext =
    				NamingContextExtHelper.narrow(rootContext.resolve_str(System.getProperty("maca_cs.naming.rootContext")+"/"+
    						System.getProperty("maca_cs.naming.authenticatorsSubContext")));
    		}
    		catch (org.omg.CosNaming.NamingContextPackage.NotFound exc) {
    			rootContext.bind_new_context(rootContext.to_name(System.getProperty("maca_cs.naming.rootContext")+"/"+
    					System.getProperty("maca_cs.naming.authenticatorsSubContext")));
    		}
    		try {
    			subContext =
    				NamingContextExtHelper.narrow(rootContext.resolve_str(System.getProperty("maca_cs.naming.rootContext")+"/"+
    						System.getProperty("maca_cs.naming.authorizersSubContext")));
    		}
    		catch (org.omg.CosNaming.NamingContextPackage.NotFound exc) {
    			rootContext.bind_new_context(rootContext.to_name(System.getProperty("maca_cs.naming.rootContext")+"/"+
    					System.getProperty("maca_cs.naming.authorizersSubContext")));
    		}
    	}
    	catch (org.omg.CORBA.COMM_FAILURE exc) {
    		System.out.println("Advertencia: servio de nomes nao econtrado - "+exc.getMessage());
    		log.println(java.util.logging.Level.WARNING, "maca_cs",
    				"maca_cs.AuthenticatorServer", "getRootNamingContext",
					"Falha: problemas no servio de nomes - "+exc.getMessage());
    	}
    	catch (org.omg.CORBA.UserException exc) {
    		System.out.println("Advertencia: problemas no servio de nomes - "+exc.getMessage());
    		log.println(java.util.logging.Level.WARNING, "maca_cs",
    				"maca_cs.AuthenticatorServer", "getRootNamingContext",
					"Falha: problemas no servio de nomes - "+exc.getMessage());
    	}
    	catch (org.omg.CORBA.SystemException exc) {
    		System.out.println("Advertencia: problemas no servio de nomes - "+exc.getMessage()+" "+exc.completed);
    		log.println(java.util.logging.Level.WARNING, "maca_cs",
    				"maca_cs.AuthenticatorServer", "getRootNamingContext",
					"Falha: servio de nomes no encontrado - "+exc.getMessage()+" "+exc.completed);
    	}
    	catch (Exception exc) {
    		System.out.println("Falha: problemas no servio de nomes - "+exc.getMessage());
    		log.println(java.util.logging.Level.SEVERE, "maca_cs",
    				"maca_cs.AuthenticatorServer", "getRootNamingContext",
					"Falha: servio de nomes no encontrado - "+exc.getMessage());
    	}
    }
    return rootContext;
  }

  public org.omg.PortableServer.Servant getAuthenticator(LogWriter aLog, org.omg.CORBA.ORB anORB, POA aPOA) {
    return  new org.omg.maca_cs.SecurityLevel2.PrincipalAuthenticatorPOATie(
                                 new PrincipalAuthenticator(aLog, anORB, aPOA));

  }

  public org.omg.PortableServer.Servant getAccessDecision(LogWriter aLog, org.omg.CORBA.ORB anORB, POA aPOA) {
    return new org.omg.maca_cs.DfResourceAccessDecision.AccessDecisionPOATie(
                                  new AccessDecision(aLog, anORB, aPOA));

  }

  public org.omg.PortableServer.Servant getMonitor(LogWriter aLog, org.omg.CORBA.ORB anORB, POA aPOA) {
    return new maca_cs.MonitorPOATie(new MonitorImpl(aLog));
  }

  public org.omg.maca_cs.SecurityLevel2.PrincipalAuthenticator principalAuthenticatorNarrow(org.omg.CORBA.Object obj)  throws java.lang.Exception  {
    return org.omg.maca_cs.SecurityLevel2.PrincipalAuthenticatorHelper.narrow(obj);
  }

  public org.omg.maca_cs.DfResourceAccessDecision.AccessDecision accessDecisionNarrow(org.omg.CORBA.Object obj)  throws java.lang.Exception  {
    return org.omg.maca_cs.DfResourceAccessDecision.AccessDecisionHelper.narrow(obj);
  }

  public maca_cs.Monitor monitorNarrow(org.omg.CORBA.Object obj)  throws java.lang.Exception  {
    return maca_cs.MonitorHelper.narrow(obj);
  }

  public AuthenticatorServer (String[] args) throws Exception {
    Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
    this.loadProperties();
    Statistics.init(this.statsLog, this.maca_cs_timer);
    this.maca_cs_timer.schedule(new GarbageColectorTimer(), 60 * 1000 * 1, 60 * 1000 * 1);

    orb = org.omg.CORBA.ORB.init(args, System.getProperties());

    NamingContextExt rootContext = this.getRootNamingContext(log, orb);

    POA     rootPOA   = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
    POA     pa_POA    = this.createPrincipalAuthenticatorPOA(rootPOA);
    POA     ado_POA   = this.createAccessDecisionPOA(rootPOA);
    POA monitor_POA   = this.createMonitorPOA(rootPOA);
    POA     creds_POA = this.createCredentialsPOA(rootPOA);

    String pa_Id = "pa";
    org.omg.PortableServer.Servant pa_Tie = this.getAuthenticator(log, orb, rootPOA);
    pa_POA.activate_object_with_id(pa_Id.getBytes(), pa_Tie);
    pa = this.principalAuthenticatorNarrow(pa_POA.id_to_reference(pa_Id.getBytes()));

    String ado_Id ="ado";
    org.omg.PortableServer.Servant ado_Tie = this.getAccessDecision(log, orb, rootPOA);
    ado_POA.activate_object_with_id(ado_Id.getBytes(), ado_Tie);
    ado = this.accessDecisionNarrow(ado_POA.id_to_reference(ado_Id.getBytes()));

    String monitor_Id ="monitor";
    org.omg.PortableServer.Servant monitor_Tie = this.getMonitor(log, orb, rootPOA);
    monitor_POA.activate_object_with_id(monitor_Id.getBytes(), monitor_Tie);
    monitor = this.monitorNarrow(monitor_POA.id_to_reference(monitor_Id.getBytes()));

    rootPOA.the_POAManager().activate();

    System.out.println(pa_Id+" "+orb.object_to_string(pa));
    File file = new File(IOR_OUTPUT_DIR + new String(pa_Id)+".ior");
    file.delete();
    file.deleteOnExit();
    OutputStream outIORStream = new BufferedOutputStream(new FileOutputStream(file, false));
    outIORStream.write(orb.object_to_string(pa).getBytes());
    outIORStream.close();

    System.out.println(ado_Id+" "+orb.object_to_string(ado));
    file = new File(IOR_OUTPUT_DIR + new String(ado_Id)+".ior");
    file.delete();
    file.deleteOnExit();
    outIORStream = new BufferedOutputStream(new FileOutputStream(file, false));
    outIORStream.write(orb.object_to_string(ado).getBytes());
    outIORStream.close();

    System.out.println(monitor_Id+" "+orb.object_to_string(monitor));
    file = new File(IOR_OUTPUT_DIR + new String(monitor_Id)+".ior");
    file.delete();
    file.deleteOnExit();
    outIORStream = new BufferedOutputStream(new FileOutputStream(file, false));
    outIORStream.write(orb.object_to_string(monitor).getBytes());
    outIORStream.close();


    file = new File(IOR_OUTPUT_DIR + "maca_cs.iors");
    file.delete();
    file.deleteOnExit();
    outIORStream =
        new BufferedOutputStream(new FileOutputStream(file, false));
    outIORStream.write((pa_Id+orb.object_to_string(pa) + ';').getBytes());
    outIORStream.write((ado_Id+orb.object_to_string(ado) + ';').getBytes());
    outIORStream.write((monitor_Id+orb.object_to_string(monitor) + ';').getBytes());
    outIORStream.close();


    if (rootContext != null) {
      String pa_Name_Context = System.getProperty("maca_cs.naming.rootContext")+"/"+
                               System.getProperty("maca_cs.naming.authenticatorsSubContext")+"/"+
                               System.getProperty("maca_cs.naming.PrincipalAuthenticator");
      rootContext.rebind(rootContext.to_name(pa_Name_Context), pa);
      System.out.println(pa_Name_Context+" "+"disponvel no servio de nomes");

      String ado_Name_Context = System.getProperty("maca_cs.naming.rootContext")+"/"+
                                System.getProperty("maca_cs.naming.authorizersSubContext")+"/"+
                                System.getProperty("maca_cs.naming.AccessDecisionObject");
      rootContext.rebind(rootContext.to_name(ado_Name_Context), ado);
      System.out.println(ado_Name_Context+" "+"disponvel no servio de nomes");

      String monitor_Name_Context = System.getProperty("maca_cs.naming.rootContext")+"/"+
                                    System.getProperty("maca_cs.naming.adminSubContext")+"/"+
                                    System.getProperty("maca_cs.naming.Monitor");
      rootContext.rebind(rootContext.to_name(monitor_Name_Context), monitor);
      System.out.println(monitor_Name_Context+" "+"disponvel no servio de nomes");
    }
    this.loadContexts();
  }

  public void run() {
    System.out.println("MACA_CS iniciado.");
    log.println(java.util.logging.Level.INFO, "maca_cs",
                "maca_cs.AuthenticatorServer", "run",
                "MACA_CS iniciado.");
    orb.run();
  }

  public void onServiceStop() {
    orb.shutdown(false);
    System.out.println("MACA_CS encerrado.");
    log.println(java.util.logging.Level.INFO, "maca_cs",
                "maca_cs.AuthenticatorServer", "onServiceStop",
                "MACA_CS encerrado.");
    log.close();
    System.exit(0);
  }

  public int timeToWait()
  {
    return 0;
  }

  public static void main(String[] args) {
    try {
      AuthenticatorServer authoServer = new AuthenticatorServer(args);
      ServiceStopper.stop(authoServer);
      authoServer.run();
    }
    catch (Exception exc) {
      exc.printStackTrace();
      log.println(java.util.logging.Level.SEVERE, "maca_cs",
                  "maca_cs.AuthenticatorServer", "main",
                  "MACA_CS no iniciado - Falha: "+exc.getMessage());
      log.close();
      System.out.println("MACA_CS no iniciado - Falha: "+exc.getMessage());
      System.exit(-1);
    }
  }
}