package maca_cs;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import maca_ca.*;
import java.text.*;
import java.util.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import org.xml.sax.helpers.*;
import org.omg.maca_cs.Security.*;
import org.omg.maca_cs.TimeBase.*;
import org.omg.maca_cs.SecurityLevel2.*;
import org.omg.maca_cs.DfResourceAccessDecision.*;

public class MonitorSaxParser extends DefaultHandler {
  //Constantes
  static private final String LINE_END = System.getProperty("line.separator");
  static private final DateFormat   DF = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss.SSS z");

  //Variaveis estticas privadas
  static private                  Writer  out = new OutputStreamWriter(System.out);
  static private  SAXParserFactory    factory = SAXParserFactory.newInstance();

  //Variveis privadas
  private String indentString = "    "; // Amount to indent
  private int     indentLevel = 0;
  private Vector elementStack = new Vector();
  private StringWriter  result = new StringWriter();
  private org.omg.maca_cs.SecurityLevel2.Credentials creds = null;

  private boolean hasAccess(String resourceName, String operation) throws Exception {
    if (creds != null) {
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      AttributeType[] attrTypes = {new AttributeType(extFamily, AccessId.value)};
      SecAttribute[] attrs = creds.get_attributes(attrTypes);
      ResourceNameComponent[] resNameCompts = {new ResourceNameComponent("Aplicacoes", resourceName)};
      ResourceName resName = new ResourceName("127.0.0.1", resNameCompts);
      return AuthenticatorServer.ado.access_allowed(resName, operation, attrs);
    }
    return false;
  }

  private class ElementNode {
    String             uri = "";
    String           lName = "";
    String           qName = "";
    Attributes       attrs = new AttributesImpl();
    String   errorMessages = "";
    String warningMessages = "";
    String        messages = "";
    StringWriter    result = new StringWriter();

    public ElementNode(String aUri, String aLName, String aQName, Attributes anAttrs) {
      uri   = aUri;
      lName = aLName;
      qName = aQName;
      attrs = anAttrs;
    }

    public void AddErrorMessage(String errorMessage) {
      errorMessages = errorMessages + errorMessage + ".";
    }

    public void AddWarningMessage(String warningMessage) {
      warningMessages = warningMessages + warningMessage + ".";
    }

    public void AddMessage(String message) {
      messages = messages + message + ".";
    }

  }

  //Mtodos privados
  private Vector getUsersKeys(String uid) {
    Vector keys = new Vector();
    if (uid == null) {
      synchronized (AuthenticatorServer.users) {
        try {
          Enumeration enum = AuthenticatorServer.users.keys();
          while (enum.hasMoreElements()) {
            keys.add(enum.nextElement());
          }
        }
        catch (Exception exc) {
        }
      }
    }
    else {
      if (AuthenticatorServer.users.containsKey(uid)) {
        keys.add(uid);
      }
    }
    return keys;
  }

  private void listActiveUsers(Attributes attrs) {
    try {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      String returningFields = attrs.getValue("returningFields");
      String uid             = attrs.getValue("uid");
      Vector keys = this.getUsersKeys(uid);
      for (int n = 0; n < keys.size(); n++) {
        UserHolder userHolder = (UserHolder) AuthenticatorServer.users.get(keys.get(n));
        if (userHolder != null) {
          elem.result.write("<user>" + LINE_END);
          User             user = userHolder.getUser();
          user.ldapEntryToXML(returningFields, elem.result, false);
          user.rolesToXML(new String[]{"cn"}, elem.result);
          userHolder.sessionsToXML(elem.result);
          elem.result.write("</user>" + LINE_END);
        }
      }
      elem.AddMessage(((keys.size() == 0)?"Nenhum usurio":((keys.size() == 1)?"1 usurio":keys.size()+" usurios")));
    }
    catch (Exception exc) {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      elem.AddErrorMessage("** Erro gerando XML: " +
                           exc.getMessage());
    }
  }

  private void closeUserSession(Attributes attrs) {
    try {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      String uid = attrs.getValue("uid");
      UserHolder userHolder = (UserHolder) AuthenticatorServer.users.get(uid);
      if (userHolder != null) {
        String sessionnumber = attrs.getValue("sessionnumber");
        if (sessionnumber != null) {
          Credentials creds = (Credentials) userHolder.getSession(sessionnumber);
          if (creds != null) {
            creds.destroy();
            elem.AddMessage("Sesso "+sessionnumber+" do usurio '"+uid+"' encerrada com sucesso");
          }
          else {
            elem.AddErrorMessage("Nmero da sesso do usurio '"+uid+"' invlido");
          }
        }
        else {
          elem.AddErrorMessage("Nmero da sesso do usurio '"+uid+"' no informado");
        }
      }
      else {
        elem.AddErrorMessage("Usurio '"+uid+"' no possui sesses abertas");
      }
    }
    catch (Exception exc) {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      elem.AddErrorMessage("** Erro gerando XML: " +
                           exc.getMessage());
    }
  }

  private void resetUserCache(Attributes attrs) {
    try {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      String uid = attrs.getValue("uid");
      UserHolder userHolder = (UserHolder) AuthenticatorServer.users.get(uid);
      if (userHolder != null) {
        User user = userHolder.getUser();
        if (user != null) {
          user.resetCache();
          elem.AddMessage("Cache do usurio '"+uid+"' limpo com sucesso");
        }
        else {
          elem.AddErrorMessage("Usurio '"+uid+"' invlido");
        }
      }
      else {
        elem.AddErrorMessage("Usurio '"+uid+"' no possui sesses abertas");
      }
    }
    catch (Exception exc) {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      elem.AddErrorMessage("** Erro gerando XML: " +
                           exc.getMessage());
    }
  }

  private void listCurrentStatistics(Attributes attrs) {
    try {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      Statistics.currentStatisticsToXML(elem.result);
      elem.AddMessage("valores obtidos em "+DF.format((new Date())));
    }
    catch (Exception exc) {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      elem.AddErrorMessage("** Erro gerando XML: " +
                           exc.getMessage());
    }
  }

  private void executeRequestAction(String action, Attributes attrs) throws Exception {
    if (attrs != null) {
      String actionName = attrs.getValue(action);
      result.write("<response action=\""+actionName+"\"");
      ElementNode elem = (ElementNode) elementStack.lastElement();
      if ("ListarAtivos;Usuarios;Monitor;MACA".equalsIgnoreCase(actionName)) {
        if (this.hasAccess(actionName, "execuo")) {
          //Aumenta o valor inicial do buffer, proporcional ao nmero de usurios
          elem.result = new StringWriter((AuthenticatorServer.users.size() + 1) *  1024);
          listActiveUsers(attrs);
        }
        else {
          elem.AddErrorMessage("Acesso ao recurso '"+actionName+"' no autorizado");
        }
      }
      else if ("EncerrarSessao;Usuarios;Monitor;MACA".equalsIgnoreCase(actionName)) {
        if (this.hasAccess(actionName, "execuo")) {
          closeUserSession(attrs);
        }
        else {
          elem.AddErrorMessage("Acesso ao recurso '"+actionName+"' no autorizado");
        }
      }
      else if ("LimparCache;Usuarios;Monitor;MACA".equalsIgnoreCase(actionName)) {
        if (this.hasAccess(actionName, "execuo")) {
          resetUserCache(attrs);
        }
        else {
          elem.AddErrorMessage("Acesso ao recurso '"+actionName+"' no autorizado");
        }
      }
      else if ("ListarValoresCorrentes;Estatisticas;Monitor;MACA".equalsIgnoreCase(actionName)) {
        if (this.hasAccess(actionName, "execuo")) {
          listCurrentStatistics(attrs);
        }
        else {
          elem.AddErrorMessage("Acesso ao recurso '"+actionName+"' no autorizado");
        }
      }
      else {
        elem.AddErrorMessage("Ao '"+actionName+"' invlida!");
      }
    }
  }

  private void endRequest() {
    ElementNode elem = (ElementNode) elementStack.lastElement();
    if (!"".equalsIgnoreCase(elem.errorMessages)) {
      result.write(" status=\"not_ok\"" +
                   " errormessage=\""+elem.errorMessages+"\"");
    }
    else {
      result.write(" status=\"ok\"" +
                   " message=\""+elem.messages+"\"");
    }
    if (!"".equalsIgnoreCase(elem.warningMessages)) {
      result.write(" warnings=\""+elem.warningMessages+"\"");
    }
    result.write(">" + LINE_END);
    result.getBuffer().append(elem.result.getBuffer());
    result.write("</response>" + LINE_END);
  }

  private void startRequest(Attributes attrs) throws Exception {
    if (attrs != null) {
      if (attrs.getLength() > 0) {
        String aName = attrs.getLocalName(0); // Attr name
        if ("".equals(aName)) {
          aName = attrs.getQName(0);
        }
        if ("action".equals(aName)) {
            executeRequestAction(aName, attrs);
          }
        }
    }
  }

  //Mtodos pblicos
  public String parser(org.omg.maca_cs.SecurityLevel2.Credentials aCreds, String request) throws Exception {
    this.creds = aCreds;
    try {
      SAXParser parser = factory.newSAXParser();
      parser.parse(new InputSource(new StringReader(request)), this);
      return result.toString();
    }
    finally {
      this.creds = null;
    }
  }

  public void startDocument() throws SAXException {
    result = new StringWriter((AuthenticatorServer.users.size() + 1) *  1024);
    result.write("<?xml version='1.0' encoding='ISO-8859-1'?>" + LINE_END);
    result.write("<maca_cs version=\"3.1\">" + LINE_END);

    nl();
    nl();
    emit("START DOCUMENT");
    nl();
    emit("<?xml version='1.0' encoding='UTF-8'?>");
  }

  public void endDocument() throws SAXException {
    result.write("</maca_cs>" + LINE_END);

    nl(); emit("END DOCUMENT");
    try {
        nl();
        out.flush();
    }
    catch (IOException e) {
        throw new SAXException("I/O error", e);
    }
  }

  public void characters(char[] buf, int offset, int len) throws SAXException {
    String s = new String(buf, offset, len);
    if (!s.trim().equals("")) { nl(); emit("CHARS:   "+s); }
  }

  public void endElement(String uri, String lName, String qName) throws SAXException {
    String eName = lName; // element name
    if ("".equals(eName)) eName = qName; // namespaceAware = false

    nl(); emit("END_ELM: </"+eName+">");

    if (eName.equals("request")) {
      endRequest();
    }
    elementStack.remove(elementStack.size() - 1);
    indentLevel--;
  }

  public void startElement(String uri, String lName, String qName, Attributes attrs) throws SAXException {
    try {
        indentLevel++;
        String eName = lName; // element name
        if ("".equals(eName)) eName = qName; // namespaceAware = false
        elementStack.add(new ElementNode(uri, lName, qName, attrs));
        if (eName.equals("request")) {
          startRequest(attrs);
        }

        nl(); emit("ELEMENT: <"+eName);

        if (attrs != null) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String aName = attrs.getLocalName(i); // Attr name
                if ("".equals(aName)) aName = attrs.getQName(i);
                nl(); emit("   ATTR: "+aName+"\t\""+attrs.getValue(i)+"\"");
            }
        }
        if (attrs.getLength() > 0) nl(); emit(">");
    }
    catch (Exception exc) {
      throw new SAXException(exc);
    }
  }

  public void error(SAXParseException err) throws SAXException {
    try {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      elem.AddErrorMessage("** Erro interpretando XML"
        + ", linha " + err.getLineNumber()
        + ", coluna " + err.getColumnNumber()
        + ", uri " + err.getSystemId());
    }
    catch (Exception exc) {
      throw err;
    }
  }

  public void fatalError(SAXParseException err) throws SAXException {
    try {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      elem.AddErrorMessage("** Erro fatal interpretando XML"
        + ", linha " + err.getLineNumber()
        + ", coluna " + err.getColumnNumber()
        + ", uri " + err.getSystemId());
    }
    catch (Exception exc) {
      throw err;
    }
  }

  public void warning(SAXParseException err) throws SAXException {
    try {
      ElementNode elem = (ElementNode) elementStack.lastElement();
      elem.AddWarningMessage("** Advertncia interpretando XML"
        + ", linha " + err.getLineNumber()
        + ", coluna " + err.getColumnNumber()
        + ", uri " + err.getSystemId());
    }
    catch (Exception exc) {
      throw err;
    }
  }

  //===========================================================
  // Utility Methods ...
  //===========================================================

  // Wrap I/O exceptions in SAX exceptions, to
  // suit handler signature requirements
  private void emit(String s) throws SAXException { /*
    try {
        out.write(s);
        out.flush();
    }
    catch (IOException e) {
        throw new SAXException("I/O error", e);
    } */
  }

  // Start a new line
  // and indent the next line appropriately
  private void nl() throws SAXException { /*
    try {
        out.write(LINE_END);
        for (int i=0; i < indentLevel; i++) out.write(indentString);
    }
    catch (IOException e) {
        throw new SAXException("I/O error", e);
    }*/
  }

}