package maca_cs;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import maca_ca.*;
import java.util.*;
import java.text.*;
import javax.naming.*;
import org.omg.PortableServer.*;
import javax.naming.directory.*;

public class PrincipalAuthenticator implements org.omg.maca_cs.SecurityLevel2.PrincipalAuthenticatorOperations {
  public static final int CLEAR_TEXT_PWD = 0;
  private static final DateFormat     df = new SimpleDateFormat("dd/MM/yyyy 's' HH:mm:ss", new Locale("pt", "BR"));
  static private final int    MAX_SESSIONS = Integer.parseInt(System.getProperty("maca_cs.user.maxSessionsPerUser"));


  private org.omg.CORBA.ORB   _orb = null;
  private POA                 _poa = null;
  private POA           _creds_POA = null;
  private POA             _ado_POA = null;
  private LogWriter           _log = new LogWriter(new PrintWriter(System.out, true));

  private String getUserRole(org.omg.maca_cs.Security.SecAttribute[] privileges) {

    for (int n = 0; n < privileges.length; n++) {
      if ((privileges[n].attribute_type.attribute_family.family_definer == 0) && // OMG famlily
          (privileges[n].attribute_type.attribute_family.family == 1) && // privilege attributes; family = 1
          (privileges[n].attribute_type.attribute_type == org.omg.maca_cs.Security.Role.value)) {
        return new String(privileges[n].value);
      }
    }
    return "";
  }

  public PrincipalAuthenticator(LogWriter aLog, org.omg.CORBA.ORB anORB, POA aPOA) {
    this._orb = anORB;
    this._poa = aPOA;
    if (aLog != null) {
      this._log = aLog;
    }
    try {
      this._creds_POA  = _poa.find_POA("creds_POA", false);
      this._ado_POA    = _poa.find_POA("ado_POA", false);
    }
    catch (org.omg.PortableServer.POAPackage.AdapterNonExistent exc) {
      _log.println(java.util.logging.Level.SEVERE, "maca_cs",
                  "maca_cs.PrincipalAuthenticator", "PrincipalAuthenticator",
                  "Falha na inicializalo: "+exc.getMessage());
    }
  }

  public User createUser(LogWriter aLog, String serverAndPort, String userRoot,
                         String userRDN, String ldap_pwd,
                         contexts.ContextExpressionInterpreter anInterpreter) throws java.lang.Exception {
    String isCached = System.getProperty("maca_cs.user.cache");
    if ((isCached != null) && isCached.equalsIgnoreCase("true")) {
      return new CachedUser(aLog, serverAndPort, userRoot, userRDN, ldap_pwd, anInterpreter);
    }
    else {
      return new User(aLog, serverAndPort, userRoot, userRDN, ldap_pwd, anInterpreter);
    }
  }

  public Credentials createCredentials(LogWriter aLog, org.omg.CORBA.ORB anORB, POA aPoa, String userName) throws java.lang.Exception {
    return new Credentials(aLog, anORB, aPoa, userName);
  }

  public org.omg.maca_cs.Security.AuthenticationStatus authenticate (int method,
                                                             java.lang.String security_name,
                                                             byte[] auth_data,
                                                             org.omg.maca_cs.Security.SecAttribute[] privileges,
                                                             org.omg.maca_cs.SecurityLevel2.CredentialsHolder creds,
                                                             org.omg.maca_cs.Security.OpaqueHolder continuation_data,
                                                             org.omg.maca_cs.Security.OpaqueHolder auth_specific_data) {
    continuation_data.value  = new byte[0];
    auth_specific_data.value = new byte[0];
    if (method == CLEAR_TEXT_PWD) {
      security_name = security_name.toLowerCase().trim();
      String uid = "uid="+security_name;
      if (auth_data.length > 0) {
        try {
          String userRole = this.getUserRole(privileges);
          String userPwd  = new String(auth_data);
          UserHolder _userHolder = null;
          User             _user = null;
          synchronized (this) {
              _userHolder = (UserHolder) maca_cs.AuthenticatorServer.users.get(security_name);
              if (_userHolder != null) {      //Usurio j possui sesses abertas, logo basta criar uma nova sesso (credencial)
                _user = _userHolder.getUser();
                if (!_user.hasPassword(userPwd)) {
                  Statistics.newUnsuccessfulAuthentication();
                  throw new AuthenticationException("LDAP: error code 49 - Invalid Credentials");
                }
              }
              else {  //Cria um usurio 
                if ((AuthenticatorServer.MAX_USERS > 0) &&
                    (AuthenticatorServer.users.size() >= AuthenticatorServer.MAX_USERS)) {
                  throw new Exception("MACA_CS: ["+AuthenticatorServer.MAX_USERS+"] - nmero mximo de usurios simultneos alcanado.");
                }
                else {
                  contexts.ContextExpressionInterpreter inter = null;
                  try {
                    inter = new contexts.ContextExpressionInterpreter(false);
                  }
                  catch (Exception except) {
                    _log.println(java.util.logging.Level.SEVERE, security_name,
                                 "maca_cs.PrincipalAuthenticator", "authenticate",
                                 "Falha na criao do interpretador de contextos do usurio: "+except.getMessage());
                  }
                  _user       = this.createUser(_log,
                                                maca_cs.AuthenticatorServer.SERVER_AND_PORT,
                                                maca_cs.AuthenticatorServer.ROOT,
                                                uid,
                                                userPwd,
                                                inter);
                  _userHolder = new UserHolder(_user);
                  if (inter != null) {
                    inter.loadContext("userCtx", new UserContext(_user));
                    inter.loadContext("netCtx", new NetContext(this._orb, this._ado_POA.id_to_reference("ado".getBytes())));
                    inter.loadContext("ldapCtx", new LdapContext(_user.getDirContextHolder()));
                    inter.loadContext(maca_cs.AuthenticatorServer.contexts);
                  }
                  maca_cs.AuthenticatorServer.users.put(security_name, _userHolder);
                }
              }  //Cria uma sesso (credencial) para o usurio
              if ((MAX_SESSIONS > 0) &&
                  (_userHolder.getSessionsNumber() >= MAX_SESSIONS)) {
                throw new Exception("Proibida a criao de credencial ["+_userHolder.getUser().getUserID()+"][]"+": nmero mximo simultneo de sesses '"+MAX_SESSIONS+"' deste usurio alcanado.");
              }
              Credentials credImpl = this.createCredentials(_log, _orb, _poa, security_name);
              creds.value = credImpl.activateCredentials();
              if (!userRole.equalsIgnoreCase("")) {
                _user.setActiveRole(userRole);
              }
              _log.println(java.util.logging.Level.INFO, _user.getUserID()+" :: "+_user.getActiveRolesName(),
                           "maca_cs.PrincipalAuthenticator", "authenticate",
                           "Login efetuado - "+_userHolder.getSessionsNumber()+" sesses abertas");
              Statistics.newSuccessfulAuthetication();
              if (_user.passwordMustChange()) {
                String message = "Sua senha DEVE ser trocada agora";
                continuation_data.value  = "passwordMustChange: true".getBytes();
                auth_specific_data.value = ((new String(auth_specific_data.value)) +
                                            (auth_specific_data.value.length > 0?" - ":"") +  message).getBytes();
                _log.println(java.util.logging.Level.WARNING, security_name,
                             "maca_cs.PrincipalAuthenticator", "authenticate",  message);
                _userHolder.lock(); //S desbloqueia com a alterao da senha
                return org.omg.maca_cs.Security.AuthenticationStatus.SecAuthContinue;
              }
              else if (_user.isPasswordExpired()) {
              	String message = "Sua senha expirou em "+
				                  df.format(_user.getPasswordAttributeDate("passwordExpirationTime")) +
								  " e DEVE ser trocada agora";
                continuation_data.value  = "passwordMustChange: true".getBytes();
                auth_specific_data.value = ((new String(auth_specific_data.value)) +
                                            (auth_specific_data.value.length > 0?" - ":"") +  message).getBytes();
                _log.println(java.util.logging.Level.WARNING, security_name,
                             "maca_cs.PrincipalAuthenticator", "authenticate", message);
                _userHolder.lock(); //S desbloqueia com a alterao da senha
                return org.omg.maca_cs.Security.AuthenticationStatus.SecAuthContinue;
              }
              else if (_user.isToWarningPasswordExpiration()) {
                auth_specific_data.value = ("Sua senha expirar em "+
                		                    df.format(_user.getPasswordAttributeDate("passwordExpirationTime")) +
											" e deve ser trocada o quanto antes").getBytes();
                _user.setPasswordExpirationWarned(true);
              }
              return org.omg.maca_cs.Security.AuthenticationStatus.SecAuthSuccess;
          }
        }
        catch (org.omg.PortableServer.POAPackage.AdapterNonExistent exc) {
          auth_specific_data.value = exc.getMessage().getBytes();
          _log.println(java.util.logging.Level.SEVERE, security_name,
                       "maca_cs.PrincipalAuthenticator", "authenticate",
                       exc.getMessage());
        }
        catch (org.omg.PortableServer.POAPackage.WrongPolicy exc) {
          auth_specific_data.value = exc.getMessage().getBytes();
          _log.println(java.util.logging.Level.SEVERE, security_name,
                       "maca_cs.PrincipalAuthenticator", "authenticate",
                       exc.getMessage());
        }
        catch (org.omg.PortableServer.POAPackage.ObjectNotActive exc) {
          auth_specific_data.value = exc.getMessage().getBytes();
          _log.println(java.util.logging.Level.SEVERE, security_name,
                       "maca_cs.PrincipalAuthenticator", "authenticate",
                       exc.getMessage());
        }
        catch (org.omg.PortableServer.POAPackage.ServantAlreadyActive exc) {
          auth_specific_data.value = exc.getMessage().getBytes();
          _log.println(java.util.logging.Level.SEVERE, security_name,
                       "maca_cs.PrincipalAuthenticator", "authenticate",
                       exc.getMessage());
        }
        catch (javax.naming.AuthenticationNotSupportedException exc) {
            auth_specific_data.value = ("A conta do usurio '"+security_name+"' est bloqueada. "+
            		                    "Entre em contato com o suporte de informtica da instituio.").getBytes();
            _log.println(java.util.logging.Level.WARNING, security_name,
                         "maca_cs.PrincipalAuthenticator", "authenticate",
                         exc.toString(true));
        }
        catch (javax.naming.AuthenticationException exc) {
            auth_specific_data.value = ("Login ou senha do usurio invlidos.").getBytes();
            _log.println(java.util.logging.Level.WARNING, security_name,
                         "maca_cs.PrincipalAuthenticator", "authenticate",
                         exc.toString(true));
        }
        catch (NamingException exc) {
          auth_specific_data.value = exc.toString(true).getBytes();
          _log.println(java.util.logging.Level.WARNING, security_name,
                       "maca_cs.PrincipalAuthenticator", "authenticate",
                       exc.toString(true));
        }
        catch (Exception e) {
          auth_specific_data.value = e.getMessage().getBytes();
          _log.println(java.util.logging.Level.SEVERE, security_name,
                       "maca_cs.PrincipalAuthenticator", "authenticate",
                       e.getMessage());
        }
        Statistics.newUnsuccessfulAuthentication();
        return org.omg.maca_cs.Security.AuthenticationStatus.SecAuthFailure;
      }
      else {
        auth_specific_data.value = "Senha no fornecida".getBytes();
        _log.println(java.util.logging.Level.WARNING, security_name,
                     "maca_cs.PrincipalAuthenticator", "authenticate",
                     "Senha no fornecida");
        Statistics.newUnsuccessfulAuthentication();
        return org.omg.maca_cs.Security.AuthenticationStatus.SecAuthFailure;
      }
    }
    else {
      Statistics.newUnsuccessfulAuthentication();
      _log.println(java.util.logging.Level.WARNING, security_name,
                   "maca_cs.PrincipalAuthenticator", "authenticate",
                   "Mtodo de autenticao no suportado - "+method);
    }
    Statistics.newUnsuccessfulAuthentication();
    return org.omg.maca_cs.Security.AuthenticationStatus.SecAuthFailure;
  }


  public org.omg.maca_cs.Security.AuthenticationStatus continue_authentication (byte[] response_data,
                                                                        org.omg.maca_cs.SecurityLevel2.Credentials creds,
                                                                        org.omg.maca_cs.Security.OpaqueHolder continuation_data,
                                                                        org.omg.maca_cs.Security.OpaqueHolder auth_specific_data) {
    try {
      continuation_data.value  = new byte[0];
      auth_specific_data.value = new byte[0];
      creds.refresh();
      return org.omg.maca_cs.Security.AuthenticationStatus.SecAuthSuccess;
    }
    catch (Exception e) {
      _log.println(java.util.logging.Level.SEVERE, "maca_cs",
                   "maca_cs.PrincipalAuthenticator", "continue_authentication",
                   e.getMessage());
    }
    return org.omg.maca_cs.Security.AuthenticationStatus.SecAuthFailure;
  }

}