package maca_cs;

import java.net.*;
import java.util.*;
import java.io.*;

public class ServiceStopper extends Thread  {

  class Stopper {
    Stopable stopable;
    Stopper next = null;
    Stopper( Stopable stopable ) {
      this.stopable = stopable;
    }
  }

  private static ServiceStopper singleton = null;
  private static Stopper stopper = null;

  public static int PORT = 8375;

  private ServiceStopper() {
    String port = System.getProperty("maca_cs.service.stop.port", Integer.toString(PORT) );
    PORT = Integer.parseInt( port );
  }

  private static synchronized ServiceStopper get() {
    if ( singleton == null ) {
      singleton = new ServiceStopper();
      singleton.setDaemon( true );
      singleton.start();
    }
    return singleton;
  }


  static public boolean stop( Stopable stopable ) {
    return get()._stop( stopable );
  }

  protected synchronized boolean _stop(Stopable stopable) {
    if (stopable == null)
      return false;

    if (stopper == null) {
      stopper = new Stopper(stopable);
    }
    else {
      Stopper newStopper = new Stopper(stopable);
      newStopper.next = stopper;
      stopper = newStopper;
    }
    return true;
  }


  public void run() {
    ServerSocket ss = null;
    try {
      ss = new ServerSocket( PORT );
    }
    catch (Exception e) {
        e.printStackTrace();
        System.exit(1);
    }
    Socket s = null;
    System.out.println( "ServiceStopper: escutando na porta " + PORT );
    do {
      try {
        if ( s != null )
          s.close();
        s = ss.accept();
      }
      catch (Exception e) {
          e.printStackTrace();
      }
    }
    while ( !s.getInetAddress().equals( s.getLocalAddress() ) );
    stopSingle();
  }

  protected int timeToWait()
  {
    Stopper s = stopper;
    if ( s != null )
      return s.stopable.timeToWait();
    return 1;
  }

  protected void stopSingle()
  {
    while( stopper != null ) {
      try {
        stopper.stopable.onServiceStop();
      } catch (Throwable e ) {
        e.printStackTrace();
      }
      stopper = stopper.next;
    }
  }
}