package maca_cs;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import java.util.*;
import maca_ca.LogWriter;

class Statistics {
  //Variveis estticas privadas
  static private final String LINE_END = System.getProperty("line.separator");
  //Mdias
  private static float   meanThreads = (float) 0.0; //Mdia de threads por minuto
  private static float     meanUsers = (float) 0.0; //Mdia de usurios com sesses simultneas abertas por minuto

  //Acumuladores totais at o momento
  private static long  acumulAutheSuc = 0; //Quantidade acumulada total de autenticaes bem sucedidas
  private static long acumulAutheFail = 0; //Quantidade acumulada total de autenticaes mal sucedidas
  private static long  acumulAuthoSuc = 0; //Quantidade acumulada total de autorizaoes bem sucedidas
  private static long acumulAuthoFail = 0; //Quantidade acumulada total de autorizaoes mal sucedidas

  //Valores correntes do ltimo minuto inteiro, isto , sem contar o minuto atual
  private static long  currentAutheSuc = 0; //Quantidade corrente de autenticaes bem sucedidas no ltimo minuto
  private static long currentAutheFail = 0; //Quantidade corrente de autenticaes mal sucedidas no ltimo minuto
  private static long  currentAuthoSuc = 0; //Quantidade corrente de autorizaoes bem sucedidas no ltimo minuto
  private static long currentAuthoFail = 0; //Quantidade corrente de autorizaoes mal sucedidas no ltimo minuto

  //Outras variveis
  private static LogWriter         log = new LogWriter(new PrintWriter(System.out, true));
  private static java.util.Timer timer = new java.util.Timer(true);
  private static long        numEvents = 1; //Quantidade de eventos para clculo das mdia, em geral
                                            //vai representar a quantidade de minutos decorridos
                                            //desde a inicializao do servidor

  static class CountingTimer extends java.util.TimerTask {
    private static long  lastAcumulAutheSuc = 0; //Quantidade acumulada total de autenticaes bem sucedidas at o ltimo minuto
    private static long lastAcumulAutheFail = 0; //Quantidade acumulada total de autenticaes mal sucedidas at o ltimo minuto
    private static long  lastAcumulAuthoSuc = 0; //Quantidade acumulada total de autorizaoes bem sucedidas at o ltimo minuto
    private static long lastAcumulAuthoFail = 0; //Quantidade acumulada total de autorizaoes mal sucedidas at o ltimo minuto
    public void run() {
      newCountingTimerEvent(this);
    }
  }

  static class LogWriterTimer extends java.util.TimerTask {
    public void run() {
      try {
        StringWriter  sw = new StringWriter();
        currentStatisticsToXML(sw);
        log.println(java.util.logging.Level.INFO, "maca_cs",
                    "maca_cs.LogWriterTimer", "run",
                    sw.toString(), false);
        sw.close();
      }
      catch (Exception exc) {
        log.println(java.util.logging.Level.SEVERE, "maca_cs",
                    "maca_cs.LogWriterTimer", "run",
                    exc.getMessage());
      }
    }
  }

  //Mtodos Privados
  private static synchronized void newCountingTimerEvent(CountingTimer aThis) {
    try {
      currentAutheSuc  = acumulAutheSuc - aThis.lastAcumulAutheSuc;
      currentAutheFail = acumulAutheFail - aThis.lastAcumulAutheFail;
      currentAuthoSuc  = acumulAuthoSuc - aThis.lastAcumulAuthoSuc;
      currentAuthoFail = acumulAuthoFail - aThis.lastAcumulAuthoFail;

      aThis.lastAcumulAutheSuc  = acumulAutheSuc;
      aThis.lastAcumulAutheFail = acumulAutheFail;
      aThis.lastAcumulAuthoSuc  = acumulAuthoSuc;
      aThis.lastAcumulAuthoFail = acumulAuthoFail;

      meanUsers        = (meanUsers * (numEvents - 1) + AuthenticatorServer.users.size()) / numEvents;
      meanThreads      = (meanThreads * (numEvents - 1) + Thread.activeCount()) / numEvents;

      numEvents     += 1;
    }
    catch (Exception exc) {
      aThis.lastAcumulAutheSuc  = 0;
      acumulAutheSuc            = 0;
      aThis.lastAcumulAutheFail = 0;
      acumulAutheFail           = 0;
      aThis.lastAcumulAuthoSuc  = 0;
      acumulAuthoSuc            = 0;
      aThis.lastAcumulAuthoFail = 0;
      acumulAuthoFail           = 0;
      numEvents                 = 1;
      log.println(java.util.logging.Level.SEVERE, "maca_cs",
                  "maca_cs.Statistics", "newCountingTimerEvent",
                  exc.getMessage());
    }
  }

  //Mtodos pblicos
  public static void init() {
    init(null, null);
  }

  public static void init(LogWriter aLog, java.util.Timer aTimer) {
    if (aLog != null) {
      log   = aLog;
    }
    if (aTimer != null) {
      timer = aTimer;
    }
    timer.schedule(new CountingTimer(), 60 * 1000 * 1, 60 * 1000 * 1);
    timer.schedule(new LogWriterTimer(),
                   60 * 1000 * 1,
                   60 * 1000 * Integer.parseInt(System.getProperty("maca_cs.log.StatisticsRefreshTime")));
  }

  public static synchronized void currentStatisticsToXML(Writer result) throws IOException {
    result.write("<currentstatistics>");
    result.write("<attribute name=\"Total de autenticacoes\" unit=\"autenticacoes\">");
    result.write("<value><![CDATA["+getTotalAuthenticationsAcumulated()+"]]></value>");
    result.write("</attribute>");
    result.write("<attribute name=\"Autenticacoes no ultimo minuto\" unit=\"autenticacoes\">");
    result.write("<value><![CDATA["+getTotalCurrentAuthentications()+"]]></value>");
    result.write("</attribute>");
    result.write("<attribute name=\"Media de autenticacoes\" unit=\"autenticacoes/min\">");
    result.write("<value><![CDATA["+getMeanTotalAuthentications()+"]]></value>");
    result.write("</attribute>");
    result.write("<attribute name=\"Total de autorizacoes\" unit=\"autorizacoes\">");
    result.write("<value><![CDATA["+getTotalAuthorizationsAcumulated()+"]]></value>");
    result.write("</attribute>");
    result.write("<attribute name=\"Autorizacoes no ultimo minuto\" unit=\"autorizacoes\">");
    result.write("<value><![CDATA["+getTotalCurrentAuthorizations()+"]]></value>");
    result.write("</attribute>");
    result.write("<attribute name=\"Media de autorizacoes\" unit=\"autorizacoes/min\">");
    result.write("<value><![CDATA["+getMeanTotalAuthorizations()+"]]></value>");
    result.write("</attribute>");
    result.write("<attribute name=\"Usuarios correntes\" unit=\"usuarios\">");
    result.write("<value><![CDATA["+getCurrentUsers()+"]]></value>");
    result.write("</attribute>");
    result.write("<attribute name=\"Media de usuarios correntes\" unit=\"usuarios/min\">");
    result.write("<value><![CDATA["+getMeanUsers()+"]]></value>");
    result.write("</attribute>");
    result.write("<attribute name=\"Threads correntes\" unit=\"threads\">");
    result.write("<value><![CDATA["+getCurrentThreads()+"]]></value>");
    result.write("</attribute>");
    result.write("<attribute name=\"Media de threads correntes\" unit=\"threads/min\">");
    result.write("<value><![CDATA["+getMeanThreads()+"]]></value>");
    result.write("</attribute>");
    result.write("</currentstatistics>");
  }

  public static synchronized void newSuccessfulAuthetication() {
    try {
      acumulAutheSuc += 1;
    }
    catch (Exception exc) {
      acumulAutheSuc = 0;
      log.println(java.util.logging.Level.SEVERE, "maca_cs",
                  "maca_cs.Statistics", "newSuccessfulAuthetication",
                  exc.getMessage());
    }
  }

  public static synchronized void newUnsuccessfulAuthentication() {
    try {
      acumulAutheFail += 1;
    }
    catch (Exception exc) {
      acumulAutheFail = 0;
      log.println(java.util.logging.Level.SEVERE, "maca_cs",
                  "maca_cs.Statistics", "newUnsuccessfulAuthentication",
                  exc.getMessage());
    }
  }

  public static synchronized void newSuccessfulAuthorization() {
    try {
      acumulAuthoSuc += 1;
    }
    catch (Exception exc) {
      acumulAuthoSuc = 0;
      log.println(java.util.logging.Level.SEVERE, "maca_cs",
                  "maca_cs.Statistics", "newSuccessfulAuthorization",
                  exc.getMessage());
    }
  }

  public static synchronized void newUnsuccessfulAuthorization() {
    try {
      acumulAuthoFail += 1;
    }
    catch (Exception exc) {
      acumulAuthoFail = 0;
      log.println(java.util.logging.Level.SEVERE, "maca_cs",
                  "maca_cs.Statistics", "newUnsuccessfulAuthorization",
                  exc.getMessage());
    }
  }

  public static synchronized float getMeanSuccessfulAuthentications() throws java.lang.ArithmeticException {
    return acumulAutheSuc / numEvents;
  }

  public static synchronized float getMeanUnSuccessfulAuthentications() throws java.lang.ArithmeticException {
    return acumulAutheFail / numEvents;
  }

  public static synchronized float getMeanTotalAuthentications() throws java.lang.ArithmeticException {
    return (float) getTotalAuthenticationsAcumulated() / (float) numEvents;
  }

  public static synchronized long getSuccessfulAuthenticationsAcumulated() {
    return acumulAutheSuc;
  }

  public static synchronized long getCurrentSuccessfulAuthentications() {
    return currentAutheSuc;
  }

  public static synchronized long getUnSuccessfulAuthenticationsAcumulated() {
    return acumulAutheFail;
  }

  public static synchronized long getCurrentUnSuccessfulAuthentications() {
    return currentAutheFail;
  }

  public static synchronized long getTotalAuthenticationsAcumulated() throws java.lang.ArithmeticException {
    return acumulAutheSuc + acumulAutheFail;
  }

  public static synchronized long getTotalCurrentAuthentications() throws java.lang.ArithmeticException {
    return currentAutheSuc + currentAutheFail;
  }

  public static synchronized float getMeanSuccessfulAuthorizations()  throws java.lang.ArithmeticException {
    return acumulAuthoSuc / numEvents;
  }

  public static synchronized float getMeanUnSuccessfulAuthorizations() throws java.lang.ArithmeticException {
    return acumulAuthoFail / numEvents;
  }

  public static synchronized float getMeanTotalAuthorizations() throws java.lang.ArithmeticException {
    return (float) getTotalAuthorizationsAcumulated() / (float) numEvents;
  }

  public static synchronized long getSuccessfulAuthorizationsAcumulated() {
    return acumulAuthoSuc;
  }

  public static synchronized long getCurrentSuccessfulAuthorizations() {
    return currentAuthoSuc;
  }

  public static synchronized long getUnSuccessfulAuthorizationsAcumulated() {
    return acumulAuthoFail;
  }

  public static synchronized long getCurrentUnSuccessfulAuthorizations() {
    return currentAuthoFail;
  }

  public static synchronized long getTotalAuthorizationsAcumulated() throws java.lang.ArithmeticException {
    return acumulAuthoSuc + acumulAuthoFail;
  }

  public static synchronized long getTotalCurrentAuthorizations() throws java.lang.ArithmeticException {
    return currentAuthoSuc + currentAuthoFail;
  }

  public static synchronized float getMeanUsers() {
    return meanUsers;
  }

  public static synchronized long getCurrentUsers() {
    return AuthenticatorServer.users.size();
  }

  public static synchronized float getMeanThreads() {
    return meanThreads;
  }

  public static synchronized long getCurrentThreads() {
    return Thread.activeCount();
  }

}