package maca_cs;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import maca_ca.*;
import java.io.*;
import java.util.*;
import java.text.*;

public class UserHolder {
  //Constantes
  static private final String     LINE_END = System.getProperty("line.separator");
  static private final DateFormat       DF = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss.SSS z");

  //Variveis privadas
  private User          user = null;
  private Hashtable sessions = new Hashtable();
  private boolean     locked = false;


  protected synchronized int openNewSession(Credentials creds) throws Exception {
    this.sessions.put(creds.getUniqueIdentifier(), creds);
    return this.sessions.size();
  }

  protected synchronized int closeSession(Credentials creds) {
    Object obj = this.sessions.remove(creds.getUniqueIdentifier());
    if (obj == null) {
      System.out.println("Falha em closeNewSession() ["+user.getUserID()+"][]"+": objeto no removido "+creds.getUniqueIdentifier());
      AuthenticatorServer.log.println(java.util.logging.Level.SEVERE, user.getUserID(),
                                      "maca_cs.UserHolder", "closeNewSession",
                                      "objeto 'Credentials' no removido - "+creds.getUniqueIdentifier());
    }
    return this.sessions.size();
  }
  
  protected void lock() {
  	locked = true;
  }

  protected void unlock() {
  	locked = false;
  }

  protected boolean isLocked() {
  	return locked;
  }

  public UserHolder(User aUser) throws Exception {
    if (aUser != null) {
      this.user = aUser;
    }
    else {
      throw new Exception("Parametro 'User aUser' criando objeto UserHolder no pode ser nulo!");
    }
  }

  public User getUser() {
    return this.user;
  }

  public int getSessionsNumber() {
    return this.sessions.size();
  }

  public Credentials getSession(String uniqueId) {
    return (Credentials) this.sessions.get(uniqueId);
  }

  public synchronized void sessionsToXML(Writer result) throws IOException {
    result.write("<sessions>" + LINE_END);
    Enumeration enum = this.sessions.keys();
    while (enum.hasMoreElements()) {
      Object key = enum.nextElement();
      Credentials creds = (Credentials) this.sessions.get(key);
      if (creds != null) {
        result.write("<session number=\""+key+"\""+
                             " creationtimestamp=\""+DF.format(new Date(creds.getCreationTimeStamp()))+"\">" + LINE_END);
        result.write("</session>" + LINE_END);
      }
    }
    result.write("</sessions>" + LINE_END);
  }

}